/*======================================================================================================================
  (c) Copyright OMRON Europe BV 2011
  All Right Reserved
  OMRON Development & Engineering
  -----------------------------------------------------------------------------------------------------------------*//**
  @file

  Register definitions for Micrel KSZ8864 switch

  @pre -
*//*--------------------------------------------------------------------------------------------------------------------
      Unique identifier generated by svn:
      $Header$

======================================================================================================================*/
#ifndef __KSZ8864_H_
#define __KSZ8864_H_

/*=== Include files ==================================================================================================*/

/*=== Constant definitions ===========================================================================================*/
#define KSZ8864_CHIP_ID                                 (0x95)

#define KSZ8864_MAX_PORTS                               (4)
#define KSZ8864_MAC_LEN                                 (6)
#define KSZ8864_SMAC_TABLE_SIZE                         (32)
#define KSZ8864_SUCCESS                                 (0)
#define KSZ8864_FAIL                                    (-1)

/*--- Serial-management Interface (SMI) ------------------------------------------------------------------------------*/
/**
 * Protocol related
 */
#define KSZ8864_SMI_WRITE_CMD                                  (0x02)
#define KSZ8864_SMI_READ_CMD                                   (0x03)

/**
 * Global switch registers
 */
#define KSZ8864_SMI_CHIP_ID0                                   (0x00)
#define KSZ8864_SMI_REV_ID_START                               (0x01)
#define   KSZ8864_SMI_REV_ID_START_REV_ID(val)                   (((val) >> 1) & 0x7)
#define   KSZ8864_SMI_REV_ID_START_START_SWITCH                  (1 << 0)
#define KSZ8864_SMI_CONTROL_0_GLOBAL                           (0x02)
#define   KSZ8864_SMI_CONTROL_0_GLOBAL_LINK_CHANGE_AGE           (1 << 0)
#define   KSZ8864_SMI_CONTROL_0_GLOBAL_UNH_MODE                  (1 << 1)
#define   KSZ8864_SMI_CONTROL_0_GLOBAL_FLUSH_SMAC_TABLE          (1 << 4)
#define   KSZ8864_SMI_CONTROL_0_GLOBAL_FLUSH_DMAC_TABLE          (1 << 5)
#define   KSZ8864_SMI_CONTROL_0_GLOBAL_NEW_BACKOFF_EN            (1 << 7)
#define KSZ8864_SMI_CONTROL_1_GLOBAL                           (0x03)
#define   KSZ8864_SMI_CONTROL_1_GLOBAL_AGRESSIVE_BACKOFF_ENABLE  (1 << 0)
#define   KSZ8864_SMI_CONTROL_1_GLOBAL_FAST_AGE_ENABLE           (1 << 1)
#define   KSZ8864_SMI_CONTROL_1_GLOBAL_AGE_ENABLE                (1 << 2)
#define   KSZ8864_SMI_CONTROL_1_GLOBAL_FRAME_LENGTH_CHECK        (1 << 3)
#define   KSZ8864_SMI_CONTROL_1_GLOBAL_RX_FLOW_CTRL_DISABLE      (1 << 4)
#define   KSZ8864_SMI_CONTROL_1_GLOBAL_TX_FLOW_CTRL_DISABLE      (1 << 5)
#define   KSZ8864_SMI_CONTROL_1_GLOBAL_2K_PACKET_SUPPORT         (1 << 6)
#define   KSZ8864_SMI_CONTROL_1_GLOBAL_PASS_ALL_FRAMES           (1 << 7)
#define KSZ8864_SMI_CONTROL_2_GLOBAL                           (0x04)
#define   KSZ8864_SMI_CONTROL_2_GLOBAL_DISABLE_MAX_PACKET_CHECK  (1 << 1)
#define   KSZ8864_SMI_CONTROL_2_GLOBAL_HUGE_PACKET_SUPPORT       (1 << 2)
#define   KSZ8864_SMI_CONTROL_2_GLOBAL_NO_EXCESSIVE_COL_DROP     (1 << 3)
#define   KSZ8864_SMI_CONTROL_2_GLOBAL_FLOWCTL_BACKP_FAIR_MODE   (1 << 4)
#define   KSZ8864_SMI_CONTROL_2_GLOBAL_BACK_PRESSURE_MODE        (1 << 5)
#define   KSZ8864_SMI_CONTROL_2_GLOBAL_STORM_EXCLUDE_MULTICAST   (1 << 6)
#define   KSZ8864_SMI_CONTROL_2_GLOBAL_DISCARD_PVLAN_MISMATCH    (1 << 7)
#define KSZ8864_SMI_CONTROL_3_GLOBAL                           (0x05)
#define KSZ8864_SMI_CONTROL_4_GLOBAL                           (0x06)
#define   KSZ8864_SMI_CONTROL_4_GLOBAL_BROADC_STORM_MASK_HI      (0x07 << 0)
#define   KSZ8864_SMI_CONTROL_4_GLOBAL_REPLACE_WITH_PORT_VID     (1 << 3)
#define   KSZ8864_SMI_CONTROL_4_GLOBAL_PORT4_SPEED_10BT          (1 << 4)
#define   KSZ8864_SMI_CONTROL_4_GLOBAL_PORT4_FLOW_CONTROL        (1 << 5)
#define   KSZ8864_SMI_CONTROL_4_GLOBAL_PORT4_HALF_DUPLEX         (1 << 6)
#define   KSZ8864_SMI_CONTROL_4_GLOBAL_PORT4_HD_BACKPRESSURE     (1 << 7)
#define KSZ8864_SMI_CONTROL_5_GLOBAL                           (0x07)
#define   KSZ8864_SMI_CONTROL_5_GLOBAL_BROADC_STORM_MASK_LO      (0xFF << 0)
#define KSZ8864_SMI_CONTROL_6_GLOBAL                           (0x08)
#define KSZ8864_SMI_CONTROL_7_GLOBAL                           (0x09)
#define KSZ8864_SMI_CONTROL_8_GLOBAL                           (0x0A)
#define KSZ8864_SMI_CONTROL_9_GLOBAL                           (0x0B)
#define   KSZ8864_SMI_CONTROL_9_GLOBAL_SMI_SAMPLE_EDGE           (1 << 0)
#define   KSZ8864_SMI_CONTROL_9_GLOBAL_LED_MODE                  (1 << 1)
#define   KSZ8864_SMI_CONTROL_9_GLOBAL_PHY_POWER_SAVE            (1 << 3)
#define   KSZ8864_SMI_CONTROL_9_GLOBAL_RMII_SW4_CLK_EDGE         (1 << 6)
#define   KSZ8864_SMI_CONTROL_9_GLOBAL_RMII_SW3_CLK_EDGE         (1 << 7)
#define KSZ8864_SMI_CONTROL_10_GLOBAL                          (0x0C)
#define   KSZ8864_SMI_CONTROL_10_GLOBAL_TAIL_TAG_ENABLE          (1 << 1)
#define KSZ8864_SMI_CONTROL_11_GLOBAL                          (0x0D)
#define KSZ8864_SMI_POWER_CONTROL_1_GLOBAL                     (0x0E)
#define KSZ8864_SMI_POWER_CONTROL_2_GLOBAL                     (0x0F)
#define KSZ8864_SMI_INTERRUPT_STATUS                           (0x7C)
#define   KSZ8864_SMI_INTERRUPT_STATUS_PORT2                     (1 << 2)
#define   KSZ8864_SMI_INTERRUPT_STATUS_PORT1                     (1 << 1)
#define KSZ8864_SMI_INTERRUPT_MASK                             (0x7D)
#define   KSZ8864_SMI_INTERRUPT_MASK_PORT2                       (1 << 2)
#define   KSZ8864_SMI_INTERRUPT_MASK_PORT1                       (1 << 1)
#define KSZ8864_SMI_CONTROL_12_GLOBAL                          (0x80)
#define KSZ8864_SMI_CONTROL_13_GLOBAL                          (0x81)
#define KSZ8864_SMI_CONTROL_14_GLOBAL                          (0x82)
#define KSZ8864_SMI_CONTROL_15_GLOBAL                          (0x83)
#define KSZ8864_SMI_CONTROL_16_GLOBAL                          (0x84)
#define KSZ8864_SMI_CONTROL_17_GLOBAL                          (0x85)
#define KSZ8864_SMI_CONTROL_18_GLOBAL                          (0x86)
#define KSZ8864_SMI_CONTROL_19_GLOBAL                          (0x87)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_0                     (0x90)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_1                     (0x91)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_2                     (0x92)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_3                     (0x93)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_4                     (0x94)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_5                     (0x95)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_6                     (0x96)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_7                     (0x97)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_8                     (0x98)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_9                     (0x99)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_10                    (0x9A)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_11                    (0x9B)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_12                    (0x9C)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_13                    (0x9D)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_14                    (0x9E)
#define KSZ8864_SMI_TOS_PRIORITY_CONTROL_15                    (0x9F)
#define KSZ8864_SMI_PORT3_MII_CONTROL_1                        (0xCF)
#define KSZ8864_SMI_PORT3_MII_CONTROL_2                        (0xDF)
#define   KSZ8864_SMI_PORT3_MII_CONTROL_2_MAC_MODE               (1 << 6)
#define KSZ8864_SMI_PORT4_MII_CONTROL                          (0xFF)

/*
 * Indirect access registers
 */
#define KSZ8864_SMI_MAC_ADDRESS_0                              (0x68)
#define KSZ8864_SMI_MAC_ADDRESS_1                              (0x69)
#define KSZ8864_SMI_MAC_ADDRESS_2                              (0x6A)
#define KSZ8864_SMI_MAC_ADDRESS_3                              (0x6B)
#define KSZ8864_SMI_MAC_ADDRESS_4                              (0x6C)
#define KSZ8864_SMI_MAC_ADDRESS_5                              (0x6D)
#define KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0                  (0x6E)
#define   KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_READ             (1U << 4)
#define   KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_WRITE            (0U << 4)
#define   KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_SMAC_TABLE       (0U << 2)
#define   KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_VLAN_TABLE       (1U << 2)
#define   KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_DMAC_TABLE       (2U << 2)
#define   KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_MIB_COUNTER      (3U << 2)
#define   KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_ADDR(addr)       (((addr) >> 8) & 0x03)
#define KSZ8864_SMI_INDERECT_ACCESS_CONTROL_1                  (0x6F)
#define   KSZ8864_SMI_INDERECT_ACCESS_CONTROL_1_ADDR(addr)       ((addr) & 0xFF)
#define KSZ8864_SMI_INDIRECT_DATA_8                            (0x70)
#define KSZ8864_SMI_INDIRECT_DATA_7                            (0x71)
#define KSZ8864_SMI_INDIRECT_DATA_6                            (0x72)
#define KSZ8864_SMI_INDIRECT_DATA_5                            (0x73)
#define KSZ8864_SMI_INDIRECT_DATA_4                            (0x74)
#define KSZ8864_SMI_INDIRECT_DATA_3                            (0x75)
#define KSZ8864_SMI_INDIRECT_DATA_2                            (0x76)
#define KSZ8864_SMI_INDIRECT_DATA_1                            (0x77)
#define KSZ8864_SMI_INDIRECT_DATA_0                            (0x78)

/* When reading the Static MAC table */
#define   KSZ8864_SMI_INDIRECT_DATA_7_SMAC_RD_FID(iData7)        ((iData7) >> 1)
#define   KSZ8864_SMI_INDIRECT_DATA_7_SMAC_RD_USE_FID(iData7)    (((iData7) & (1U << 0)) != 0U)
#define   KSZ8864_SMI_INDIRECT_DATA_6_SMAC_RD_OVERRIDE_PSTATE(iData6) \
                                                                 (((iData6) & (1U << 6)) != 0U)
#define   KSZ8864_SMI_INDIRECT_DATA_6_SMAC_RD_IS_VALID(iData6)   (((iData6) & (1U << 5)) != 0U)
#define   KSZ8864_SMI_INDIRECT_DATA_6_SMAC_RD_FWD_MASK(iData6)   (((iData6) >> 1) & 0x0F)

/* When writing the Static MAC table */
#define   KSZ8864_SMI_INDIRECT_DATA_7_SMAC_WR_FID(fid)           ((fid) & 0x7F)
#define   KSZ8864_SMI_INDIRECT_DATA_6_SMAC_WR_USE_FID            (1U << 7)
#define   KSZ8864_SMI_INDIRECT_DATA_6_SMAC_WR_OVERRIDE_PSTATE    (1U << 6)
#define   KSZ8864_SMI_INDIRECT_DATA_6_SMAC_WR_IS_VALID           (1U << 5)
#define   KSZ8864_SMI_INDIRECT_DATA_6_SMAC_WR_FWD_MASK(mask)     (((mask) & 0xF) << 1)

/* Generate the port bits for the forwarding mask. Ports are 1-based! */
#define   KSZ8864_SMI_INDIRECT_DATA_6_SMAC_FWD_TO_PORT(port)     (1U << (port-1))

/* When reading the Static MIB table */
#define   KSZ8864_SMI_INDIRECT_DATA_3_MIB_PPORT_OVERFLOW(iData3) (((iData3) & (1U << 7)) != 0U)
#define   KSZ8864_SMI_INDIRECT_DATA_3_MIB_PPORT_VALID(iData3)    (((iData3) & (1U << 6)) != 0U)
#define   KSZ8864_SMI_INDIRECT_DATA_MIB_PPORT_VALUE(iData3, iData2, iData1, iData0) \
                                                                 ((((iData3) & 0x3F) << 24) | ((iData2) << 16) | ((iData1) << 8) | (iData0))

/* When reading the Dynamic MAC table */
#define   KSZ8864_SMI_INDIRECT_DATA_6_DMAC_READY(iData6)         (((iData6) & 0x80) != 0U)
#define   KSZ8864_SMI_INDIRECT_DATA_7_SOURCE_PORT(iData7)        ((iData7) & 0x07)
#define   KSZ8864_SMI_INDIRECT_DATA_7_TIMESTAMP(iData7)          (((iData7) >> 3) & 0x3)
#define   KSZ8864_SMI_INDIRECT_DATA_NO_ENTRIES(iData7, iData8)   ((((iData8) & 0x80) != 0U) \
                                                                  ? 0 \
                                                                  : (((((unsigned) (iData7) >> 5) & 0x7) | \
                                                                      (((unsigned) (iData8) & 0x7f) << 3)  ) + 1U) )



/*
 * Per-port SMI registers
 */
#define KSZ8864_SMI_PORTX_CONTROL_0                            (0x10)
#define   KSZ8864_SMI_PORTX_CONTROL_0_STORM_PROTECT_ENABLE       (1 << 7)
#define   KSZ8864_SMI_PORTX_CONTROL_0_DIFFSERV_PRIO_ENABLE       (1 << 6)
#define   KSZ8864_SMI_PORTX_CONTROL_0_802_1P_PRIO_ENABLE         (1 << 5)
#define   KSZ8864_SMI_PORTX_CONTROL_0_PORT_PRIO_ENABLE_MASK      (0x3 << 3)
#define   KSZ8864_SMI_PORTX_CONTROL_0_TAG_INSERTION              (1 << 2)
#define   KSZ8864_SMI_PORTX_CONTROL_0_TAG_REMOVAL                (1 << 1)
#define KSZ8864_SMI_PORTX_CONTROL_1                            (0x11)
#define   KSZ8864_SMI_PORTX_CONTROL_1_IS_SNIFFER_PORT            (1 << 7)
#define   KSZ8864_SMI_PORTX_CONTROL_1_SNIFF_ON_RX                (1 << 6)
#define   KSZ8864_SMI_PORTX_CONTROL_1_SNIFF_ON_TX                (1 << 5)
#define   KSZ8864_SMI_PORTX_CONTROL_1_VLAN_MEMBERSHIP_MASK       (0x1f << 0)
#define KSZ8864_SMI_PORTX_CONTROL_2                            (0x12)
#define   KSZ8864_SMI_PORTX_CONTROL_2_TRANSMIT_ENABLE            (1 << 2)
#define   KSZ8864_SMI_PORTX_CONTROL_2_RECEIVE_ENABLE             (1 << 1)
#define   KSZ8864_SMI_PORTX_CONTROL_2_LEARNING_DISABLE           (1 << 0)
#define   KSZ8864_SMI_PORTX_CONTROL_2_RSTP_STATE_MASK            (KSZ8864_SMI_PORTX_CONTROL_2_TRANSMIT_ENABLE  | \
                                                                  KSZ8864_SMI_PORTX_CONTROL_2_RECEIVE_ENABLE   | \
                                                                  KSZ8864_SMI_PORTX_CONTROL_2_LEARNING_DISABLE)
#define   KSZ8864_SMI_PORTX_CONTROL_2_RSTP_STATE_DISABLE         (KSZ8864_SMI_PORTX_CONTROL_2_LEARNING_DISABLE)
#define   KSZ8864_SMI_PORTX_CONTROL_2_RSTP_STATE_BLOCKING        (KSZ8864_SMI_PORTX_CONTROL_2_LEARNING_DISABLE)
#define   KSZ8864_SMI_PORTX_CONTROL_2_RSTP_STATE_LISTENING       (KSZ8864_SMI_PORTX_CONTROL_2_LEARNING_DISABLE)
#define   KSZ8864_SMI_PORTX_CONTROL_2_RSTP_STATE_LEARNING        (0)
#define   KSZ8864_SMI_PORTX_CONTROL_2_RSTP_STATE_FORWARDING      (KSZ8864_SMI_PORTX_CONTROL_2_RECEIVE_ENABLE | \
                                                                  KSZ8864_SMI_PORTX_CONTROL_2_TRANSMIT_ENABLE)
#define KSZ8864_SMI_PORTX_CONTROL_3                            (0x13)
#define KSZ8864_SMI_PORTX_CONTROL_4                            (0x14)
#define KSZ8864_SMI_PORTX_STATUS_0                             (0x19)
#define KSZ8864_SMI_PORTX_PHY_CTRL_STAT                        (0x1A)
#define KSZ8864_SMI_PORTX_CONTROL_5                            (0x1C)
#define KSZ8864_SMI_PORTX_CONTROL_6                            (0x1D)
#define KSZ8864_SMI_PORTX_STATUS_1                             (0x1E)
#define   KSZ8864_SMI_PORTX_STATUS1_MDIX                         (1 << 7)
#define   KSZ8864_SMI_PORTX_STATUS1_AN_DONE                      (1 << 6)
#define   KSZ8864_SMI_PORTX_STATUS1_LINK_GOOD                    (1 << 5)
#define   KSZ8864_SMI_PORTX_STATUS1_LP_FLOW_CTRL                 (1 << 4)
#define   KSZ8864_SMI_PORTX_STATUS1_LP_100BT_FULL                (1 << 3)
#define   KSZ8864_SMI_PORTX_STATUS1_LP_100BT_HALF                (1 << 2)
#define   KSZ8864_SMI_PORTX_STATUS1_LP_10BT_FULL                 (1 << 1)
#define   KSZ8864_SMI_PORTX_STATUS1_LP_10BT_HALF                 (1 << 0)
#define KSZ8864_SMI_PORTX_CONTROL_7_STATUS_2                   (0x1F)
#define   KSZ8864_SMI_PORTX_CONTROL_7_PHY_LB                     (1 << 7)
#define   KSZ8864_SMI_PORTX_CONTROL_7_PHY_ISOLATE                (1 << 5)
#define   KSZ8864_SMI_PORTX_CONTROL_7_SOFT_RESET                 (1 << 4)
#define   KSZ8864_SMI_PORTX_CONTROL_7_FORCE_LINK                 (1 << 3)
#define   KSZ8864_SMI_PORTX_STATUS_2_PORT_MODE(val)              ((val) & 0x7)
#define   KSZ8864_SMI_PORTX_STATUS_2_PORT_MODE_AN_BUSY           (1)
#define   KSZ8864_SMI_PORTX_STATUS_2_PORT_MODE_10BT_HALF         (2)
#define   KSZ8864_SMI_PORTX_STATUS_2_PORT_MODE_100BT_HALF        (3)
#define   KSZ8864_SMI_PORTX_STATUS_2_PORT_MODE_10BT_FULL         (5)
#define   KSZ8864_SMI_PORTX_STATUS_2_PORT_MODE_100BT_FULL        (6)
#define KSZ8864_SMI_PORTX_CONTROL_8                            (0xB0)
#define KSZ8864_SMI_PORTX_CONTROL_9                            (0xB1)
#define KSZ8864_SMI_PORTX_CONTROL_10                           (0xB2)
#define KSZ8864_SMI_PORTX_CONTROL_11                           (0xB3)
#define KSZ8864_SMI_PORTX_CONTROL_12                           (0xB4)
#define KSZ8864_SMI_PORTX_CONTROL_13                           (0xB5)
#define KSZ8864_SMI_PORTX_RATE_CONTROL                         (0xB6)
#define KSZ8864_SMI_PORTX_INGRESS_CONTROL_1                    (0xB7)
#define KSZ8864_SMI_PORTX_INGRESS_CONTROL_2                    (0xB8)
#define KSZ8864_SMI_PORTX_INGRESS_CONTROL_3                    (0xB9)
#define KSZ8864_SMI_PORTX_INGRESS_CONTROL_4                    (0xBA)
#define KSZ8864_SMI_PORTX_EGRESS_CONTROL_1                     (0xBB)
#define KSZ8864_SMI_PORTX_EGRESS_CONTROL_2                     (0xBC)
#define KSZ8864_SMI_PORTX_EGRESS_CONTROL_3                     (0xBD)
#define KSZ8864_SMI_PORTX_EGRESS_CONTROL_4                     (0xBE)

#define KSZ8864_SMI_PORT1_OFFSET                               (0x10)
#define KSZ8864_SMI_PORT2_OFFSET                               (0x20)
#define KSZ8864_SMI_PORT3_OFFSET                               (0x30)
#define KSZ8864_SMI_PORT4_OFFSET                               (0x40)

#define KSZ8864_MII_CONTROL                                    (0x00)
#define KSZ8864_MII_STATUS                                     (0x01)
#define KSZ8864_MII_PHYID_HI                                   (0x02)
#define KSZ8864_MII_PHYID_LO                                   (0x03)
#define KSZ8864_MII_ADVERTISE                                  (0x04)
#define KSZ8864_MII_PARTNER_ABILITY                            (0x05)
#define KSZ8864_MII_SPECIAL_CONTROL_STATUS                     (0x1F)

#define KSZ8864_MII_PORT1_ADDR                                 (0x02)
#define KSZ8864_MII_PORT2_ADDR                                 (0x03)

#define KSZ8864_MIB_PORTX_RX_LO_PRIORITY_BYTES                 (0x00)
#define KSZ8864_MIB_PORTX_RX_HI_PRIORITY_BYTES                 (0x01)
#define KSZ8864_MIB_PORTX_RX_UNDERSIZE_PACKETS                 (0x02)
#define KSZ8864_MIB_PORTX_RX_FRAGMENTS                         (0x03)
#define KSZ8864_MIB_PORTX_RX_OVERSIZE_PACKETS                  (0x04)
#define KSZ8864_MIB_PORTX_RX_JABBERS                           (0x05)
#define KSZ8864_MIB_PORTX_RX_SYMBOL_ERRORS                     (0x06)
#define KSZ8864_MIB_PORTX_RX_CRC_ERRORS                        (0x07)
#define KSZ8864_MIB_PORTX_RX_ALIGNMENT_ERRORS                  (0x08)
#define KSZ8864_MIB_PORTX_RX_CONTROL_8808_PACKETS              (0x09)
#define KSZ8864_MIB_PORTX_RX_PAUSE_PACKETS                     (0x0A)
#define KSZ8864_MIB_PORTX_RX_BROADCAST_PACKETS                 (0x0B)
#define KSZ8864_MIB_PORTX_RX_MULTICAST_PACKETS                 (0x0C)
#define KSZ8864_MIB_PORTX_RX_UNICAST_PACKETS                   (0x0D)
#define KSZ8864_MIB_PORTX_RX_64_BYTE_PACKETS                   (0x0E)
#define KSZ8864_MIB_PORTX_RX_65_TO_127_BYTE_PACKETS            (0x0F)
#define KSZ8864_MIB_PORTX_RX_128_TO_255_BYTE_PACKETS           (0x10)
#define KSZ8864_MIB_PORTX_RX_256_TO_511_BYTE_PACKETS           (0x11)
#define KSZ8864_MIB_PORTX_RX_512_TO_1023_BYTE_PACKETS          (0x12)
#define KSZ8864_MIB_PORTX_RX_1024_TO_1522_BYTE_PACKETS         (0x13)
#define KSZ8864_MIB_PORTX_TX_LO_PRIORITY_BYTES                 (0x14)
#define KSZ8864_MIB_PORTX_TX_HI_PRIORITY_BYTES                 (0x15)
#define KSZ8864_MIB_PORTX_TX_LATE_COLLISIONS                   (0x16)
#define KSZ8864_MIB_PORTX_TX_PAUSE_PACKETS                     (0x17)
#define KSZ8864_MIB_PORTX_TX_BROADCAST_PACKETS                 (0x18)
#define KSZ8864_MIB_PORTX_TX_MULTICAST_PACKETS                 (0x19)
#define KSZ8864_MIB_PORTX_TX_UNICAST_PACKETS                   (0x1A)
#define KSZ8864_MIB_PORTX_TX_DEFERRED_PACKETS                  (0x1B)
#define KSZ8864_MIB_PORTX_TX_TOTAL_COLLISIONS                  (0x1C)
#define KSZ8864_MIB_PORTX_TX_EXCESSIVE_COLLISIONS              (0x1D)
#define KSZ8864_MIB_PORTX_TX_SINGLE_COLLISIONS                 (0x1E)
#define KSZ8864_MIB_PORTX_TX_MULTIPLE_COLLISIONS               (0x1F)

#define KSZ8864_MIB_PORT1_OFFSET                               (0x20)
#define KSZ8864_MIB_PORT2_OFFSET                               (0x40)
#define KSZ8864_MIB_PORT3_OFFSET                               (0x60)
#define KSZ8864_MIB_PORT4_OFFSET                               (0x80)

#define KSZ8864_MIB_IS_PORTX_REGISTER(addr)                    (((addr) >= 0x00) && ((addr) <= 0x9F))

#define KSZ8864_MIB_PACKETS_DROPPED_PORT1_TX                   (0x101)
#define KSZ8864_MIB_PACKETS_DROPPED_PORT2_TX                   (0x102)
#define KSZ8864_MIB_PACKETS_DROPPED_PORT3_TX                   (0x103)
#define KSZ8864_MIB_PACKETS_DROPPED_PORT4_TX                   (0x104)

#define KSZ8864_MIB_PACKETS_DROPPED_PORT1_RX                   (0x106)
#define KSZ8864_MIB_PACKETS_DROPPED_PORT2_RX                   (0x107)
#define KSZ8864_MIB_PACKETS_DROPPED_PORT3_RX                   (0x108)
#define KSZ8864_MIB_PACKETS_DROPPED_PORT4_RX                   (0x109)

#define KSZ8864_FORWARD_PORT1                                  (1 << 0)
#define KSZ8864_FORWARD_PORT2                                  (1 << 1)
#define KSZ8864_FORWARD_PORT3                                  (1 << 2)
#define KSZ8864_FORWARD_PORT4                                  (1 << 3)
#define KSZ8864_FORWARD_PORT_ALL                               (KSZ8864_FORWARD_PORT1 | \
                                                                KSZ8864_FORWARD_PORT2 | \
                                                                KSZ8864_FORWARD_PORT3 | \
                                                                KSZ8864_FORWARD_PORT4 )
/* Tail tag bit definitions */
#define KSZ8864_TAIL_TAG_INGRESS_PORT_MASK                     (0x07 << 1)
#define KSZ8864_TAIL_TAG_INGRESS_TO_PORT(port)                 (0x01 << (port))
#define   KSZ8864_TAIL_TAG_INGRESS_TO_PORT_NORMAL                (0x00 << 0)
#define   KSZ8864_TAIL_TAG_INGRESS_TO_PORT1                      (0x01 << 1)
#define   KSZ8864_TAIL_TAG_INGRESS_TO_PORT2                      (0x01 << 2)
#define   KSZ8864_TAIL_TAG_INGRESS_TO_PORT3                      (0x01 << 3)
#define KSZ8864_TAIL_TAG_INGRESS_QUEUE_MASK                    (0x07 << 4)
#define   KSZ8864_TAIL_TAG_INGRESS_QUEUE0                        (0x00 << 4)
#define   KSZ8864_TAIL_TAG_INGRESS_QUEUE1                        (0x01 << 4)
#define   KSZ8864_TAIL_TAG_INGRESS_QUEUE2                        (0x02 << 4)
#define   KSZ8864_TAIL_TAG_INGRESS_QUEUE3                        (0x03 << 4)
#define   KSZ8864_TAIL_TAG_INGRESS_QUEUE_WHATEVER                (0x04 << 4)
#define   KSZ8864_TAIL_TAG_INGRESS_IGNORE                        (0x01 << 7)

#define KSZ8864_TAIL_TAG_EGRESS_PORT_MASK                      (0x03 << 0)
#define   KSZ8864_TAIL_TAG_EGRESS_FROM_PORT1                     (0x01 << 0)
#define   KSZ8864_TAIL_TAG_EGRESS_FROM_PORT2                     (0x02 << 0)
#define   KSZ8864_TAIL_TAG_EGRESS_FROM_PORT3                     (0x03 << 0)
#define   KSZ8864_TAIL_TAG_EGRESS_IS_VALID(tag)                ((tag) >= KSZ8864_TAIL_TAG_EGRESS_FROM_PORT1 && \
                                                                (tag) <= KSZ8864_TAIL_TAG_EGRESS_FROM_PORT3)

/*=== Macro definitions ==============================================================================================*/
#define KSZ8864_BYTE_ALIGN(type)    __attribute__ ((packed, aligned(1))) (type)

/*=== Type definitions ===============================================================================================*/
typedef enum
{
  KSZ8864_PAUSE_FLOW_CTRL_NONE=0,
  KSZ8864_PAUSE_FLOW_CTRL_TX_ONLY,
  KSZ8864_PAUSE_FLOW_CTRL_RX_ONLY,
  KSZ8864_PAUSE_FLOW_CTRL_FULL
} T_KSZ8864_PAUSE_FLOW_CTRL;

typedef enum
{
  KSZ8864_PORT3_MODE_PHY=0,
  KSZ8864_PORT3_MODE_MAC,
} T_KSZ8864_PORT3_MODE;

typedef enum
{
  KSZ8864_MIB_COUNTER_STATUS_OK = 0,    /*< The counter value is okay */
  KSZ8864_MIB_COUNTER_STATUS_OVERFLOW,  /*< The counter value overflowed */
} T_KSZ8864_MIB_COUNTER_STATUS;

typedef enum
{
  KSZ8864_LED_MODE_LNK_ACT_0_SPEED_1 = 0,
  KSZ8864_LED_MODE_100LNK_ACT_0_DUPLEX_1
} T_KSZ8864_LED_MODE;

typedef enum
{
  KSZ8864_SMAC_INVALID = 0, /**< Static MAC address entry is valid */
  KSZ8864_SMAC_VALID        /**< Static MAC address entry is invalid */
} T_KSZ8864_SMAC_STATUS;

typedef enum
{
  KSZ8864_PORT_STATE_DONT_OVERRIDE = 0,
  KSZ8864_PORT_STATE_OVERRIDE,
} T_KSZ8864_PS_OVERRIDE;

typedef enum
{
  KSZ8864_SOURCE_PORT1 = 0,
  KSZ8864_SOURCE_PORT2,
  KSZ8864_SOURCE_PORT3,
  KSZ8864_SOURCE_PORT4,
} T_KSZ8864_SOURCE_PORT;

typedef struct
{
  unsigned char command;
  unsigned char registerNo;
} KSZ8864_BYTE_ALIGN(T_KSZ8864_TX_FRAME_HEAD);

typedef struct
{
  unsigned char control;
  unsigned char address;
} KSZ8864_BYTE_ALIGN( T_KSZ8864_INDIRECT_TRANSACT );

typedef struct
{
  unsigned char fid;            /* KSZ8864_SMI_INDIRECT_DATA_7 */
  unsigned char rules;          /* KSZ8864_SMI_INDIRECT_DATA_6 */
  unsigned char macAddress5;    /* KSZ8864_SMI_INDIRECT_DATA_5 */
  unsigned char macAddress4;    /* KSZ8864_SMI_INDIRECT_DATA_4 */
  unsigned char macAddress3;    /* KSZ8864_SMI_INDIRECT_DATA_3 */
  unsigned char macAddress2;    /* KSZ8864_SMI_INDIRECT_DATA_2 */
  unsigned char macAddress1;    /* KSZ8864_SMI_INDIRECT_DATA_1 */
  unsigned char macAddress0;    /* KSZ8864_SMI_INDIRECT_DATA_0 */
} KSZ8864_BYTE_ALIGN( T_KSZ_SMI_INDIRECT_RW_DATA );

typedef struct
{
  unsigned char entriesHi;      /* KSZ8864_SMI_INDIRECT_DATA_8 */
  unsigned char portTimeEntryLo;/* KSZ8864_SMI_INDIRECT_DATA_7 */
  unsigned char fidReady;       /* KSZ8864_SMI_INDIRECT_DATA_6 */
} KSZ8864_BYTE_ALIGN( T_KSZ_SMI_INDIRECT_DMAC_DATA_HI );

typedef struct
{
  unsigned char macAddress5;    /* KSZ8864_SMI_INDIRECT_DATA_5 */
  unsigned char macAddress4;    /* KSZ8864_SMI_INDIRECT_DATA_4 */
  unsigned char macAddress3;    /* KSZ8864_SMI_INDIRECT_DATA_3 */
  unsigned char macAddress2;    /* KSZ8864_SMI_INDIRECT_DATA_2 */
  unsigned char macAddress1;    /* KSZ8864_SMI_INDIRECT_DATA_1 */
  unsigned char macAddress0;    /* KSZ8864_SMI_INDIRECT_DATA_0 */
} KSZ8864_BYTE_ALIGN( T_KSZ_SMI_INDIRECT_DMAC_DATA_LO );

typedef struct {
  T_KSZ8864_SMAC_STATUS valid;                    /**< Is the entry valid? */
  T_KSZ8864_PS_OVERRIDE override;                 /**< Override RSTP port state? */
  unsigned              forwardMask;              /**< The bitmask defining which ports to forward to
                                                       (E.g. 0x01 = port 1, 0x05 = port 3 & 1, etc) */
  unsigned long long	macAddr;                  /**< This entry's MAC address */
} T_KSZ8864_STATIC_MAC_ENTRY;

typedef struct {
  unsigned              timestamp;                /**< 2-bit counters for internal aging */
  T_KSZ8864_SOURCE_PORT sourcePort;               /**< The source port where FID+MAC is learned */
  unsigned char         macAddr[KSZ8864_MAC_LEN]; /**< This entry's MAC address */
} T_KSZ8864_DYNAMIC_MAC_ENTRY;

/**
 * Incomplete type for switch device (Completed differently for linux driver and boot-loader)
 * NOTE: The only functions that need to know what's inside ethsw_net, are the external functions declared below. These
 * 4 functions are uniquely defined in the boot-loader and driver.
 */
struct ethsw_net;

/*=== Type definitions for events ====================================================================================*/

/*=== Macro-function definitions =====================================================================================*/

/*=== External function declarations (Must be defined where ksz8864 is included) =====================================*/
void ksz8864_writeData(         struct ethsw_net*           ks,
                                unsigned const              startReg,
                                void const* const           data,
                                unsigned const              noBytes );
void ksz8864_readData(          struct ethsw_net*           ks,
                                unsigned const              startReg,
                                void* const                 data,
                                unsigned const              noBytes );
void ksz8864_writeRegister(     struct ethsw_net*           ks,
                                unsigned const              reg,
                                unsigned const              value );
unsigned ksz8864_readRegister(  struct ethsw_net*           ks,
                                unsigned const              reg );


/*=== Inline-function definitions ====================================================================================*/
/**
 * Sets one or more bits in a switch register
 * @param[in] ks    The switch device
 * @param[in] reg     Register number
 * @param[in] bits    Mask containing bits to set
 */
__inline__ static void ksz8864_setRegBits(struct ethsw_net* ks, unsigned reg, unsigned bits)
{
  int value = ksz8864_readRegister(ks, reg);
  value |= bits;
  ksz8864_writeRegister(ks, reg, value);
}

/**
 * Sets one or more bits in a switch register
 * @param[in] ks    The switch device
 * @param[in] reg     Register number
 * @param[in] bits    Mask containing bits to set
 */
__inline__ static void ksz8864_clearRegBits(struct ethsw_net* ks, unsigned reg, unsigned bits)
{
  int value = ksz8864_readRegister(ks, reg);
  value &= ~bits;
  ksz8864_writeRegister(ks, reg, value);
}

/**
 * Reads the chip identification of the switch
 * @param[in] ks    The switch device
 */
__inline__ static unsigned ksz8864_readChipId(struct ethsw_net* ks)
{
  return ksz8864_readRegister(ks, KSZ8864_SMI_CHIP_ID0);
}

/**
 * Enables the ksz8864 switch
 * @param[in] ks    The switch device
 */
__inline__ static void ksz8864_enable(struct ethsw_net* ks)
{
  ksz8864_setRegBits(ks, KSZ8864_SMI_REV_ID_START, KSZ8864_SMI_REV_ID_START_START_SWITCH);
}

/**
 * Disables the ksz8864 switch
 * @param[in] ks    The switch device
 */
__inline__ static void ksz8864_disable(struct ethsw_net* ks)
{
  ksz8864_clearRegBits(ks, KSZ8864_SMI_REV_ID_START, KSZ8864_SMI_REV_ID_START_START_SWITCH);
}

/**
 * Sets the pause-frame flow control mode
 * @param[in] ks    The switch device
 * @param[in] mode  The mode to use for pause flow control
 */
__inline__ static void ksz8864_setPauseFlowControl(struct ethsw_net* ks, T_KSZ8864_PAUSE_FLOW_CTRL mode)
{
  switch(mode)
  {
    case KSZ8864_PAUSE_FLOW_CTRL_NONE:
    {
      ksz8864_setRegBits(ks,
                        KSZ8864_SMI_CONTROL_1_GLOBAL,
                        KSZ8864_SMI_CONTROL_1_GLOBAL_RX_FLOW_CTRL_DISABLE | KSZ8864_SMI_CONTROL_1_GLOBAL_TX_FLOW_CTRL_DISABLE);
      break;
    }
    case KSZ8864_PAUSE_FLOW_CTRL_TX_ONLY:
    {
      ksz8864_clearRegBits(ks, KSZ8864_SMI_CONTROL_1_GLOBAL, KSZ8864_SMI_CONTROL_1_GLOBAL_TX_FLOW_CTRL_DISABLE);
      ksz8864_setRegBits(  ks, KSZ8864_SMI_CONTROL_1_GLOBAL, KSZ8864_SMI_CONTROL_1_GLOBAL_RX_FLOW_CTRL_DISABLE);
      break;
    }
    case KSZ8864_PAUSE_FLOW_CTRL_RX_ONLY:
    {
      ksz8864_clearRegBits(ks, KSZ8864_SMI_CONTROL_1_GLOBAL, KSZ8864_SMI_CONTROL_1_GLOBAL_RX_FLOW_CTRL_DISABLE);
      ksz8864_setRegBits(  ks, KSZ8864_SMI_CONTROL_1_GLOBAL, KSZ8864_SMI_CONTROL_1_GLOBAL_TX_FLOW_CTRL_DISABLE);
      break;
    }
    default: /* fall-through */
    case KSZ8864_PAUSE_FLOW_CTRL_FULL:
    {
      ksz8864_clearRegBits(ks,
                          KSZ8864_SMI_CONTROL_1_GLOBAL,
                          KSZ8864_SMI_CONTROL_1_GLOBAL_RX_FLOW_CTRL_DISABLE | KSZ8864_SMI_CONTROL_1_GLOBAL_TX_FLOW_CTRL_DISABLE);
      break;
    }
  }
}

/**
 * Sets the port3 mode
 * @param[in] mode  The mode to use for port 3
 * @param[in] ks    The switch device
 */
__inline__ static void ksz8864_port3Mode(struct ethsw_net* ks, T_KSZ8864_PORT3_MODE mode)
{
  if(mode == KSZ8864_PORT3_MODE_MAC)
  {
    ksz8864_setRegBits(ks, KSZ8864_SMI_PORT3_MII_CONTROL_2, KSZ8864_SMI_PORT3_MII_CONTROL_2_MAC_MODE);
  }
  else
  {
    ksz8864_clearRegBits(ks, KSZ8864_SMI_PORT3_MII_CONTROL_2, KSZ8864_SMI_PORT3_MII_CONTROL_2_MAC_MODE);
  }
}

/**
 * Straps port 4 for 100BT full-duplex mode
 * Required to use switch as PHY for the MAC controller
 * @param[in] ks    The switch device
 */
__inline__ static void ksz8864_strapPort4(struct ethsw_net* ks)
{
  ksz8864_clearRegBits(ks,
                      KSZ8864_SMI_CONTROL_4_GLOBAL,
                      KSZ8864_SMI_CONTROL_4_GLOBAL_PORT4_SPEED_10BT |
                      KSZ8864_SMI_CONTROL_4_GLOBAL_PORT4_HALF_DUPLEX |
                      KSZ8864_SMI_CONTROL_4_GLOBAL_PORT4_FLOW_CONTROL |
                      KSZ8864_SMI_CONTROL_4_GLOBAL_PORT4_HD_BACKPRESSURE );
}

/**
 * Sets the LED mode
 * Required to use switch as PHY for the MAC controller
 * @param[in] ks    The switch device
 * @param[in] mode  The LED mode
 */
__inline__ static void ksz8864_setModeLEDs(struct ethsw_net* ks, T_KSZ8864_LED_MODE const mode)
{
  if(mode == KSZ8864_LED_MODE_LNK_ACT_0_SPEED_1)
  {
    ksz8864_clearRegBits(ks, KSZ8864_SMI_CONTROL_9_GLOBAL, KSZ8864_SMI_CONTROL_9_GLOBAL_LED_MODE);
  }
  else
  {
    ksz8864_setRegBits(ks, KSZ8864_SMI_CONTROL_9_GLOBAL, KSZ8864_SMI_CONTROL_9_GLOBAL_LED_MODE);
  }
}

/**
 * Probes for the presence of the KSZ8864
 * @param[in]   ks      The switch device
 * @return              @p 0 if the KSZ8864 was successfully probed,
 *                      @p -1 if not
 */
__inline__ static int ksz8864_probe(struct ethsw_net* ks)
{
    return (ksz8864_readChipId(ks) == KSZ8864_CHIP_ID) ? 0 : -1;
}

/**
 * Initialises and starts the switch
 * Disables pause-frame flow control, sets port 3 to MAC mode, straps port 4 for PHY mode and starts the switch
 * @param[in] ks    The switch device
 */
__inline__ static void ksz8864_start(struct ethsw_net* ks)
{
  ksz8864_setPauseFlowControl(ks, KSZ8864_PAUSE_FLOW_CTRL_NONE);
  ksz8864_port3Mode(ks, KSZ8864_PORT3_MODE_MAC);
  ksz8864_strapPort4(ks);
  ksz8864_setModeLEDs(ks, KSZ8864_LED_MODE_LNK_ACT_0_SPEED_1);
  ksz8864_enable(ks);
}

/**
 * Adds a MAC address to the static table
 * @param[in] ks            The switch device
 * @param[in] index         The MAC address index to set
 * @param[in] entry         The static MAC address entry data
 */
__inline__ static void ksz8864_addStaticMacAddr(struct ethsw_net*                       ks,
                                                unsigned const                          index,
                                                T_KSZ8864_STATIC_MAC_ENTRY const* const entry )
{
  T_KSZ_SMI_INDIRECT_RW_DATA writeData =
  {
    0U,
    ( (entry->override != KSZ8864_PORT_STATE_DONT_OVERRIDE) ? KSZ8864_SMI_INDIRECT_DATA_6_SMAC_WR_OVERRIDE_PSTATE : 0U )
      | ( (entry->valid != KSZ8864_SMAC_INVALID) ? KSZ8864_SMI_INDIRECT_DATA_6_SMAC_WR_IS_VALID : 0U )
      | KSZ8864_SMI_INDIRECT_DATA_6_SMAC_WR_FWD_MASK(entry->forwardMask),
    (unsigned int)(entry->macAddr >> 40) & 0xFF,	/* Most-significant byte of MAC address */
    (unsigned int)(entry->macAddr >> 32) & 0xFF,
    (unsigned int)(entry->macAddr >> 24) & 0xFF,
    (unsigned int)(entry->macAddr >> 16) & 0xFF,
    (unsigned int)(entry->macAddr >>  8) & 0xFF,
    (unsigned int)(entry->macAddr      ) & 0xFF,	/* Least-significant byte of MAC address */
  };

  T_KSZ8864_INDIRECT_TRANSACT transactionData =
  {
    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_WRITE
      | KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_SMAC_TABLE
      | KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_ADDR(index),
    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_1_ADDR(index),
  };

  /* Indirect access to read mode before starting */
  ksz8864_writeRegister(ks,
                        KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0,
                        KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_READ);

  /* Set-up the transaction data */
  ksz8864_writeData(ks,
                    KSZ8864_SMI_INDIRECT_DATA_7,
                    &writeData,
                    sizeof(writeData) );

  /* Start the transaction */
  ksz8864_writeData(ks,
                    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0,
                    &transactionData,
                    sizeof(transactionData) );

//  KSDBG("Wrote static MAC table entry %d: FID=0x%02x, RULES=0x%02x, MAC=0x%02x:%02x:%02x:%02x:%02x:%02x\n",
//        index, writeData.fid, writeData.rules,
//        writeData.macAddress5, writeData.macAddress4, writeData.macAddress3,
//        writeData.macAddress2, writeData.macAddress1, writeData.macAddress0);
}


/**
 * Reads a MAC address from the static table
 * @param[in]   ks      The switch device
 * @param[in]   index   The MAC address index to read
 * @param[out]  entry   The static MAC address entry data
 * @return              KSZ8864_SMAC_VALID = Valid entry
 *                      KSZ8864_SMAC_INVALID = Entry invalid (unused)
 */
__inline__  static T_KSZ8864_SMAC_STATUS ksz8864_readStaticMacAddr(struct ethsw_net*                 ks,
                                                                   unsigned const                    index,
                                                                   T_KSZ8864_STATIC_MAC_ENTRY* const entry)
{
  T_KSZ_SMI_INDIRECT_RW_DATA readData;
  T_KSZ8864_INDIRECT_TRANSACT transactionData =
  {
    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_READ
      | KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_SMAC_TABLE
      | KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_ADDR(index),
    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_1_ADDR(index),
  };

  /* Start the transaction */
  ksz8864_writeData(ks,
                    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0,
                    &transactionData,
                    sizeof(transactionData) );

  /* Set-up the transaction data */
  ksz8864_readData( ks,
                    KSZ8864_SMI_INDIRECT_DATA_7,
                    &readData,
                    sizeof(readData) );

  entry->valid       = KSZ8864_SMI_INDIRECT_DATA_6_SMAC_RD_IS_VALID(readData.rules) != 0
                       ? KSZ8864_SMAC_VALID
                       : KSZ8864_SMAC_INVALID;
  entry->override    = KSZ8864_SMI_INDIRECT_DATA_6_SMAC_RD_OVERRIDE_PSTATE(readData.rules)
                       ? KSZ8864_PORT_STATE_OVERRIDE
                       : KSZ8864_PORT_STATE_DONT_OVERRIDE;
  entry->forwardMask = KSZ8864_SMI_INDIRECT_DATA_6_SMAC_RD_FWD_MASK(readData.rules);
  entry->macAddr     =  (((unsigned long long)readData.macAddress0)      ) |
                        (((unsigned long long)readData.macAddress1) <<  8) |
                        (((unsigned long long)readData.macAddress2) << 16) |
                        (((unsigned long long)readData.macAddress3) << 24) |
                        (((unsigned long long)readData.macAddress4) << 32) |
                        (((unsigned long long)readData.macAddress5) << 40);

//  KSDBG("Read static MAC table entry %d: FID=0x%02x, RULES=0x%02x, MAC=0x%02x:%02x:%02x:%02x:%02x:%02x\n",
//        index, readData.fid, readData.rules,
//        readData.macAddress5, readData.macAddress4, readData.macAddress3,
//        readData.macAddress2, readData.macAddress1, readData.macAddress0);

  return entry->valid;
}

/**
 * Reads a MAC address from the dynamic address table
 * @param[in]     ks      The switch device
 * @param[in,out] index   Read index
 * @param[out]    entry   The dynamic MAC address entry data
 * @return                The number of entries in the dynamic MAC address table, or
 *                        @p -1 if an error occurred
 */
__inline__  static int ksz8864_readDynamicMacAddr(struct ethsw_net*                  ks,
                                                  unsigned                           index,
                                                  T_KSZ8864_DYNAMIC_MAC_ENTRY* const entry)
{
  int result;
  T_KSZ_SMI_INDIRECT_DMAC_DATA_HI readDataHi;
  T_KSZ_SMI_INDIRECT_DMAC_DATA_LO readDataLo;
  T_KSZ8864_INDIRECT_TRANSACT transactionData =
  {
    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_READ
      | KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_DMAC_TABLE
      | KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_ADDR(index),
    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_1_ADDR(index),
  };

  /* Start the transaction */
  ksz8864_writeData(ks,
                    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0,
                    &transactionData,
                    sizeof(transactionData) );

  ksz8864_readData( ks,
                    KSZ8864_SMI_INDIRECT_DATA_8,
                    &readDataHi,
                    sizeof(readDataHi) );

  /* Wait until the data is ready */
  while(KSZ8864_SMI_INDIRECT_DATA_6_DMAC_READY(readDataHi.fidReady))
  {
    readDataHi.fidReady = ksz8864_readRegister( ks, KSZ8864_SMI_INDIRECT_DATA_6 );
  }

  ksz8864_readData( ks,
                    KSZ8864_SMI_INDIRECT_DATA_5,
                    &readDataLo,
                    sizeof(readDataLo) );

  result = KSZ8864_SMI_INDIRECT_DATA_NO_ENTRIES(readDataHi.portTimeEntryLo, readDataHi.entriesHi);

  if((result > 0) && (index < (unsigned) result))
  {
    entry->timestamp   = KSZ8864_SMI_INDIRECT_DATA_7_TIMESTAMP(readDataHi.portTimeEntryLo);
    entry->sourcePort  = (T_KSZ8864_SOURCE_PORT) KSZ8864_SMI_INDIRECT_DATA_7_SOURCE_PORT(readDataHi.portTimeEntryLo);
    entry->macAddr[0]  = readDataLo.macAddress5;  /* Most-significant byte of MAC address */
    entry->macAddr[1]  = readDataLo.macAddress4;
    entry->macAddr[2]  = readDataLo.macAddress3;
    entry->macAddr[3]  = readDataLo.macAddress2;
    entry->macAddr[4]  = readDataLo.macAddress1;
    entry->macAddr[5]  = readDataLo.macAddress0;  /* Least-significant byte of MAC address */
  }
  else
  {
    result = -1;
  }
  return result;
}

/**
 * Reads a counter from the MIB table
 * @param[in] ks            Switch device
 * @param[in] offset        Offset of the counter to read
 * @param[in] value         Counter value
 * @return                  Counter status ( See @link T_KSZ8864_MIB_COUNTER_STATUS )
 */
__inline__  static T_KSZ8864_MIB_COUNTER_STATUS ksz8864_getCounterMIB(struct ethsw_net* ks,
                                                                      unsigned const    offset,
                                                                      unsigned* const   value )
{
  T_KSZ8864_MIB_COUNTER_STATUS retval = KSZ8864_MIB_COUNTER_STATUS_OK;
  T_KSZ8864_INDIRECT_TRANSACT transactionData =
  {
    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_READ
      | KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_MIB_COUNTER
      | KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0_ADDR(offset),
    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_1_ADDR(offset),
  };

  /* Start the transaction */
  ksz8864_writeData(ks,
                    KSZ8864_SMI_INDERECT_ACCESS_CONTROL_0,
                    &transactionData,
                    sizeof(transactionData) );

  if(KSZ8864_MIB_IS_PORTX_REGISTER((int) offset))
  {
    struct
    {
      unsigned char count3;            /* KSZ8864_SMI_INDIRECT_DATA_3 */
      unsigned char count2;            /* KSZ8864_SMI_INDIRECT_DATA_2 */
      unsigned char count1;            /* KSZ8864_SMI_INDIRECT_DATA_1 */
      unsigned char count0;            /* KSZ8864_SMI_INDIRECT_DATA_0 */
    } KSZ8864_BYTE_ALIGN( countData );

    /* Wait for the data to become valid */
    do
    {
      countData.count3 = ksz8864_readRegister(ks, KSZ8864_SMI_INDIRECT_DATA_3);
    } while( KSZ8864_SMI_INDIRECT_DATA_3_MIB_PPORT_VALID(countData.count3) == 0U );

    /* Read the rest */
    ksz8864_readData(ks,
                     KSZ8864_SMI_INDIRECT_DATA_2,
                     &countData.count2,
                     sizeof(countData) - sizeof(countData.count3));

    *value = KSZ8864_SMI_INDIRECT_DATA_MIB_PPORT_VALUE(countData.count3,
                                                       countData.count2,
                                                       countData.count1,
                                                       countData.count0);
    if(KSZ8864_SMI_INDIRECT_DATA_3_MIB_PPORT_OVERFLOW(countData.count3))
    {
      retval = KSZ8864_MIB_COUNTER_STATUS_OVERFLOW;
    }
  }
  else
  {
    struct
    {
      unsigned char count1;            /* KSZ8864_SMI_INDIRECT_DATA_1 */
      unsigned char count0;            /* KSZ8864_SMI_INDIRECT_DATA_0 */
    } KSZ8864_BYTE_ALIGN( countData );

    /* Read the rest */
    ksz8864_readData(ks,
                     KSZ8864_SMI_INDIRECT_DATA_2,
                     &countData,
                     sizeof(countData) );

    *value = KSZ8864_SMI_INDIRECT_DATA_MIB_PPORT_VALUE(0U, 0U,
                                                       countData.count1,
                                                       countData.count0);
  }
  return retval;
}

/**
 * Sets a number of MAC addresses to the static MAC address table
 * @param[in]   ks          The switch device
 * @param[in]   entries     Array of MAC addresses to add
 * @param[in]   entryCount  Number of elements in @p entries
 * @return                  KSZ8864_SMAC_VALID = Valid entry
 *                          KSZ8864_SMAC_INVALID = Entry invalid (unused)
 */
__inline__ static T_KSZ8864_SMAC_STATUS ksz8864_setStaticEntries(struct ethsw_net*                       ks,
                                                                 T_KSZ8864_STATIC_MAC_ENTRY const* const entries,
                                                                 unsigned const                          entryCount)
{
  T_KSZ8864_SMAC_STATUS retval = KSZ8864_SMAC_VALID;
  unsigned              entryNo;

  for(entryNo = 0; (entryNo < entryCount) && (retval == KSZ8864_SMAC_VALID); entryNo++)
  {
    if((entries[entryNo].override > 1) ||
       (entries[entryNo].forwardMask > ((1U << KSZ8864_MAX_PORTS) - 1U)) ||
       (entries[entryNo].macAddr == 0ULL) )
    {
      retval = KSZ8864_SMAC_INVALID;
    }
    else
    {
      ksz8864_addStaticMacAddr(ks, entryNo, entries + entryNo);
    }
  }
  return retval;
}

/*=== Function declarations (Interface functions) ====================================================================*/

#endif

