/*======================================================================================================================
  (c) Copyright OMRON Europe BV 2012
  All Right Reserved
  OMRON Development & Engineering
  --------------------------------------------------------------------------------------------------------------------*/
/**
  @file

  Implements the LED sub-system

*/
/*--------------------------------------------------------------------------------------------------------------------
      Unique identifier generated by svn:
      $Header$

======================================================================================================================*/

/*=== Include files ==================================================================================================*/
#include "leds.h"
#include "device.h"

/*=== Constant definitions (File scope) ==============================================================================*/

/*=== Macro definitions (File scope) =================================================================================*/

/*=== Type definitions (File scope) ==================================================================================*/

/*=== Type definitions for events (File scope) =======================================================================*/

/*=== Function declarations (File scope) =============================================================================*/

/*=== Macro-function definitions (File scope) ========================================================================*/

/*=== Data definitions (File scope) ==================================================================================*/
static T_LED const* myLeds = NULL;

/*=== Local Function definitions (File scope) ========================================================================*/

/*=== Global function definitions (Global system or Local sub-system scope) ==========================================*/
/**
 * Initialises the LED sub-system
 * @param[in] leds      Table defining the LED's (must be static)
 * @param[in] ledCount  Table size
 */
void leds_init(T_LED const* const leds, unsigned const ledCount)
{
    unsigned idx;
    myLeds = leds;
    for(idx = 0; idx < ledCount; idx++) {
        T_LED const* const led = leds + idx;
        DEVICE_setPinDirection(led->bank, led->pin, FALSE);
        leds_set(idx, led->initOn);
    }
}

/**
 * Sets a LED to on or off
 * @param[in] led   The configured LED to set
 * @param[in] on    @p TRUE = on, @p FALSE = off
 */
void leds_set(unsigned ledNo, Bool const on)
{
    if(myLeds != NULL) {
        T_LED const* const led = myLeds + ledNo;
        if(on != FALSE) {
            DEVICE_setPin(led->bank, led->pin, !led->inverted);
        } else {
            DEVICE_setPin(led->bank, led->pin, led->inverted);
        }
    }
}
