/*======================================================================================================================
  (c) Copyright OMRON Europe BV 2012
  All Right Reserved
  OMRON Development & Engineering
  --------------------------------------------------------------------------------------------------------------------*/
/**
  @file

  Implements setting the MAC address in the AM18xx EMAC peripheral

*/
/*--------------------------------------------------------------------------------------------------------------------
      Unique identifier generated by svn:
      $Header$

======================================================================================================================*/

/*=== Include files ==================================================================================================*/
#include "types.h"
#include "switch.h"
#include "otags.h"
#include "macaddrs.h"

/*=== Constant definitions (File scope) ==============================================================================*/
#define MAC_CTRL_REGS           (0x01e23000)
#define MACSRCADDRLO            (VUint32*)(MAC_CTRL_REGS + 0x1D0) /* MAC Source Address Low Bytes Register Section 18.3.3.35 */
#define MACSRCADDRHI            (VUint32*)(MAC_CTRL_REGS + 0x1D4) /* MAC Source Address High Bytes Register Section 18.3.3.36 */

/*=== Macro definitions (File scope) =================================================================================*/

/*=== Type definitions (File scope) ==================================================================================*/

/*=== Type definitions for events (File scope) =======================================================================*/

/*=== Function declarations (File scope) =============================================================================*/

/*=== Macro-function definitions (File scope) ========================================================================*/

/*=== Data definitions (File scope) ==================================================================================*/

/*=== Local Function definitions (File scope) ========================================================================*/

/*=== Global function definitions (Global system or Local sub-system scope) ==========================================*/
/**
 * Configures the device MAC addresses in the EMAC and Ethernet switch
 * @param[in]       ks
 * @param[in,out]   otagsList
 * @return          @p TRUE if successful, @p FALSE if not
 */
BOOL macaddrs_set(struct ethsw_net* ks, T_OTAG_VALUES const* const otagsList)
{
    unsigned idx;

    for(idx = 0; idx < otagsList->mac.count; idx++) {
        T_OTAG_MAC const* const             address = otagsList->mac.address[idx];
#ifdef BOOT_ETH_SWITCH    
        T_KSZ8864_STATIC_MAC_ENTRY const    entry   = {
            .valid       = KSZ8864_SMAC_VALID,
            .override    = KSZ8864_PORT_STATE_DONT_OVERRIDE,
            .forwardMask = KSZ8864_FORWARD_PORT4,
            .macAddr     = ((((unsigned long long)address->macAddrLo >>  0U) & 0xFF) << 40U) |
                           ((((unsigned long long)address->macAddrLo >>  8U) & 0xFF) << 32U) |
                           ((((unsigned long long)address->macAddrLo >> 16U) & 0xFF) << 24U) |
                           ((((unsigned long long)address->macAddrLo >> 24U) & 0xFF) << 16U) |
                           ((((unsigned long long)address->macAddrHi >>  0U) & 0xFF) <<  8U) |
                           ((((unsigned long long)address->macAddrHi >>  8U) & 0xFF) <<  0U),
        };
        ksz8864_addStaticMacAddr(ks, idx, &entry);
#endif
        if(idx == 0U) {
            *MACSRCADDRLO = (((address->macAddrHi >>  0U) & 0xFF) <<  0U) |
                            (((address->macAddrHi >>  8U) & 0xFF) <<  8U);
            *MACSRCADDRHI = (((address->macAddrLo >>  0U) & 0xFF) <<  0U) |
                            (((address->macAddrLo >>  8U) & 0xFF) <<  8U) |
                            (((address->macAddrLo >> 16U) & 0xFF) << 16U) |
                            (((address->macAddrLo >> 24U) & 0xFF) << 24U);
        }
    }
    return TRUE;
}
