/*========================================================================================================================================== 
(c) Copyright OMRON Europe BV 2012
All Right Reserved 
OMRON Development & Engineering 
-------------------------------------------------------------------------------------------------------------------------------------*//** 
@file 

<short description of the file in one sentence>. 

[Optional extended description of the file. More details] 

[@pre <pre-conditions and Environment restrictions>] 
*//*======================================================================================================================================*/ 

/*=== Include files ======================================================================================================================*/ 
#include "types.h"
#include "debug.h"
#include "leds.h"
#include "led_defs.h"
#include "crc32.h"

/*=== Constant definitions (File scope) ==================================================================================================*/ 

/*=== Macro definitions (File scope) =====================================================================================================*/ 

/*=== Type definitions (File scope) ======================================================================================================*/ 

/*=== Type definitions for events (File scope) ===========================================================================================*/ 
typedef UNS32 (*T_NEXT_VALUE)(unsigned long const index, unsigned const sequence);

/*=== Function declarations (File scope) =================================================================================================*/ 

/*=== Macro-function definitions (File scope) ============================================================================================*/ 

/*=== Data definitions (File scope) ======================================================================================================*/

/*=== Local Function definitions (File scope) ============================================================================================*/ 
static UNS32 getSequence(unsigned long const index, unsigned const sequence)
{
    return (index + sequence);
}

static UNS32 getWalkingOne(unsigned long const index, unsigned const sequence)
{
    return (1U << ((index + sequence) % 32));
}

static UNS32 getWalkingZero(unsigned long const index, unsigned const sequence)
{
    return ~getWalkingOne(index, sequence);
}

static UNS32 getSlowSequence(unsigned long const index, unsigned const sequence)
{
    crc32_calculate(sequence, &index, sizeof(index));
    return (index + sequence);
}

static UNS32 getRandom(unsigned long const index, unsigned const sequence)
{
    return crc32_calculate(sequence, &index, sizeof(index));
}

struct {
    char*        name;
    T_NEXT_VALUE value;
    unsigned     failLed;
} tests[] = {
    { "sequence numbers",      getSequence,     LED_BF },
    { "slow sequence numbers", getSlowSequence, LED_PRPHL },
    { "walking ones",          getWalkingOne,   LED_COMM },
    { "walking zeroes",        getWalkingZero,  LED_RUN },
    { "random numbers",        getRandom,       LED_RUN },
};

void mem_fill(unsigned long const start, unsigned long const size, T_NEXT_VALUE value, unsigned const sequence)
{
    UNS32*      next    = (UNS32*) start;
    unsigned    index;

    for(index = 0U; index < (size / sizeof(*next)); index++) {
        next[index] = value(index, sequence);
    }
}

int mem_verify(unsigned long const start, unsigned long const size, T_NEXT_VALUE value, unsigned const sequence)
{
    int         result  = 0;
    UNS32*      next    = (UNS32*) start;
    unsigned    index;

    for(index = 0U; index < (size / sizeof(*next)); index++) {
        UNS32 const read     = next[index];
        UNS32 const expected = value(index, sequence);
        if(read != expected) {
            DEBUG1_printString("\t\t\tfail @ ");
            DEBUG1_printHexInt(start + (index * sizeof(*next)));
            DEBUG1_printString(": ");
            DEBUG1_printHexInt(read);
            DEBUG1_printString(" != ");
            DEBUG1_printHexInt(expected);
            DEBUG1_printString(" (");
            DEBUG1_printHexInt(read ^ expected);
            DEBUG1_printString(")\r\n");
            result = -1;
        }
    }
    return result;
}

/*=== Global function definitions (Global system or Local sub-system scope) ==============================================================*/ 
void mem_stress(unsigned long const start, unsigned long const size)
{
    unsigned sequence = 0U;

    DEBUG1_printString("Stressing memory: ");
    DEBUG1_printHexInt(start);
    DEBUG1_printString("--");
    DEBUG1_printHexInt(start+size);
    DEBUG1_printString("\r\n");

    while(1) {
        unsigned i;

        DEBUG1_printString("\tsequence ");
        DEBUG1_printHexInt(sequence);
        DEBUG1_printString("\r\n");

        for(i = 0; i < ARRAY_SIZE(tests); i++) {
            DEBUG1_printString("\t\t");
            DEBUG1_printString(tests[i].name);
            DEBUG1_printString("\r\n");
            mem_fill(start, size, tests[i].value, sequence);
            if(mem_verify(start, size, tests[i].value, sequence)) {
                leds_set(tests[i].failLed, TRUE);
            }
        }
        sequence++;
    }
}

BOOL mem_test_short(unsigned long const start, unsigned long const size)
{
    volatile UNS16 readVal;
    unsigned sequence = 0U;
    unsigned bit;
    unsigned i;
    unsigned long const testSize = (size > 0x2000 ? 0x2000 : size);

    /* Low level cell-test and data test */
    while (sequence < 33) {
        for(i = 0; i < ARRAY_SIZE(tests); i++) {
            mem_fill(start, testSize, tests[i].value, sequence);
            if(mem_verify(start, testSize, tests[i].value, sequence)) {
                DEBUG1_printString("DRAM cell test failed\r\n");
                return FALSE;
            }
        }
        sequence++;
    }

    /* Address line test, write values */
    UNS16 counter = 2;
    bit = 1;
    while (size > (unsigned)(1<<bit)) {
        UNS16* const addr = (UNS16 * const)(start + (1<<bit));
        *addr = counter;
        counter++;
        bit++;
    }

    /* Address line test, read values */
    bit--; counter--;
    while (bit >= 1) {
        volatile UNS16* const addr[] = {
            (volatile UNS16* const)(start + 0),
            (volatile UNS16* const)(start + size - 2),
            (volatile UNS16* const)(start + (1<<bit))
        };

        readVal = *(addr[0]);
        readVal = *(addr[1]);
        readVal = *(addr[2]);

        if (readVal != counter) {
            DEBUG1_printString("DRAM address test failed\r\n");
            return FALSE;
        }

        bit--;
        counter--;
    }

    return TRUE;
}

/* END OF FILE */ 
