/*======================================================================================================================
  (c) Copyright OMRON Europe BV 2011
  All Right Reserved
  OMRON Development & Engineering
  -----------------------------------------------------------------------------------------------------------------*//**
  @file

  This module parses the Omron product tag structure (O-Tag)
  O-tags are used to convey production information and Linux kernel configure parameters to the system. O-tags can be
  defined per boot-image (a board can have multiple images to support different modes of operation), or for all images.
  The number of tags is only limited by the reserved memory space to store them. Unknown tags are skipped and not parsed.

*//*--------------------------------------------------------------------------------------------------------------------
      Unique identifier generated by svn:
      $Header$

======================================================================================================================*/

#ifndef __OTAGS_H                         /* Only include this file once   */
#define __OTAGS_H                         /* If not defined, define it now */

/*=== Include files ==================================================================================================*/
#include "types.h"

/*=== Constant definitions ===========================================================================================*/
/**
 * Maximum number of MAC addresses that can be parsed
 */
#define MAX_NO_MAC_ADDRS (10)

/*=== Macro definitions ==============================================================================================*/

/*=== Type definitions ===============================================================================================*/
/**
 * Structure containing the members for an MAC O-tag
 */
typedef struct {
  UNS16     portId;     /**< The port number for which the MAC-address needs to be set */
  UNS16     macAddrHi;  /**< The two high bytes of the MAC-address */
  UNS32     macAddrLo;  /**< The four low bytes of the MAC-address */
} T_OTAG_MAC;

/**
 * Status bitmask
 */
typedef unsigned T_OTAG_ERROR_MASK;

/**
 * The status for parsing an O-tag list.
 */
typedef enum
{
    OTAG_NO_LIST                  = (1<<0),   //!< Nothing found at the memory location
    OTAG_CRC_ERRORS               = (1<<1),   //!< List contained 1 or more CRC error(s)
    OTAG_TOO_MANY_MACS            = (1<<2),   //!< An unexpected number of MAC addresses (> MAX_NO_MAC_ADDRS)
    OTAG_TOO_MANY_LINUX_ADD_CMDS  = (1<<3),   //!< An unexpected number of Linux command lines (>1)
    OTAG_TOO_MANY_LINUX_OVR_CMDS  = (1<<4),   //!< An unexpected number of Linux command lines (>1)
    OTAG_TOO_MANY_UPC_STRINGS     = (1<<5),   //!< An unexpected number of UPC strings (>1)
} T_OTAG_ERRORS;

typedef struct {
    T_OTAG_ERROR_MASK       errors;
    struct {
        unsigned            count;                      /**< No MAC addresses */
        T_OTAG_MAC const*   address[MAX_NO_MAC_ADDRS];  /**< Parsed MAC addresses */
    } mac;
    struct {
        unsigned            length;
        UNS8 const*         string;
    } upc;
    struct {
        unsigned            length;
        char const*         string;
    } linuxCmdOverride;
    struct {
        unsigned            length;
        char const*         string;
    } linuxCmdAdd;
} T_OTAG_VALUES;

#define OTAG_VALUES_INIT    { 0U,                                                                   \
                              {0U, {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL}},   \
                              {0U, NULL},                                                           \
                              {0U, NULL},                                                           \
                              {0U, NULL} }

/*=== Type definitions for events ====================================================================================*/

/*=== Macro-function definitions =====================================================================================*/

/*=== Inline-function definitions ====================================================================================*/

/*=== Function declarations (Interface functions) ====================================================================*/
T_OTAG_VALUES* otags_parseList(void const* const buffer, unsigned const imageNr);

#endif

