/*======================================================================================================================
  (c) Copyright OMRON Europe BV 2012
  All Right Reserved
  OMRON Development & Engineering
  -----------------------------------------------------------------------------------------------------------------*//**
  @file

  Implements Linux reboot2 system-call data parsing for the AM18xx-OMN platform.

*//*--------------------------------------------------------------------------------------------------------------------
      Unique identifier generated by svn:
      $Header$

======================================================================================================================*/

/*=== Include files ==================================================================================================*/
#include "debug.h"
#include "types.h"
#include "crc32.h"
#include "string.h"
#include "rebootcmd.h"

/*=== Constant definitions (File scope) ==============================================================================*/
/**
 * Reset persistent data block magic numbers
 */
#define OMN_LINUX_MAGIC1        (0x666f7220)
#define OMN_LINUX_MAGIC2        (0x626f6f74)

/*=== Macro definitions (File scope) =================================================================================*/

/*=== Type definitions (File scope) ==================================================================================*/

typedef struct
{
  UNS32     magic1;
  UNS32     magic2;
  UNS32     checksum;
  char      cmd[256];
} T_RESET_CMD_DATA;

/*=== Type definitions for events (File scope) =======================================================================*/

/*=== Function declarations (File scope) =============================================================================*/

/*=== Macro-function definitions (File scope) ========================================================================*/

/*=== Data definitions (File scope) ==================================================================================*/

/*=== Local Function definitions (File scope) ========================================================================*/

/*=== Global function definitions (Global system or Local sub-system scope) ==========================================*/
/**
 * Reads the boot command sent by the kernel
 * @param[in,out]   rebootRAMend    End of the reboot persistent RAM used to communicate the data across
 *                                  reboots
 * @return          The parsed reboot command if successful
 *                  @p NULL if not
 */
T_REBOOTCMD const* rebootcmd_parse(void* const rebootRAMend)
{
    T_RESET_CMD_DATA* const cmdData = (T_RESET_CMD_DATA* const ) (rebootRAMend - sizeof(T_RESET_CMD_DATA));
    UNS32 const             crc32   = crc32_calculate(0U, cmdData->cmd, sizeof(cmdData->cmd));
    static T_REBOOTCMD      result  = { -1, NULL };
    static T_RESET_CMD_DATA cmdDataCopy;

    if ((cmdData->magic1 != OMN_LINUX_MAGIC1) || (cmdData->magic2 != OMN_LINUX_MAGIC2)) {
        return NULL;
    } else if (crc32 != cmdData->checksum) {
        DEBUG1_printString("error: Reboot command CRC fail\n\r");
        return NULL;
    } else {
        static const char imageTag[]        = "image=";
        unsigned const    imageTagNoChars   = sizeof(imageTag) - 1;
        unsigned const    imageNoNoChars    = 1U;
        static const char separator[]       = ":";
        unsigned const    separatorNoChars  = sizeof(separator) - 1;

        DEBUG1_printString("Parsing reboot command\n\r");

        /* Clear the reboot RAM (Or it could likely still be there on the next reboot) */
        cmdDataCopy = *cmdData;
        memset(cmdData, 0, sizeof(*cmdData));

        if(strncmp(cmdDataCopy.cmd, imageTag, imageTagNoChars) == 0) {
            result.bootImageNo = (int) cmdDataCopy.cmd[imageTagNoChars] - '0';

            if( (strncmp(cmdDataCopy.cmd + imageTagNoChars + imageNoNoChars, separator, separatorNoChars) != 0) ||
                (result.bootImageNo < 0) ||
                (result.bootImageNo > 9) ) {
                DEBUG1_printString("error: Unexpected image specifier\n\r");
                return NULL;
            }
            DEBUG1_printString("\timage=");
            DEBUG1_printHexChar(result.bootImageNo);
            DEBUG1_printString("\n\r");
            result.cmdlineAdd = cmdDataCopy.cmd + imageTagNoChars + imageNoNoChars + separatorNoChars;
        } else {
            result.cmdlineAdd = cmdDataCopy.cmd;
        }
        DEBUG1_printString("\tcmdlineAdd=");
        DEBUG1_printString(result.cmdlineAdd);
        DEBUG1_printString("\n\r");
    }
    return &result;
}

/* END OF FILE */
