/*
 * TI Booting and Flashing Utilities
 *
 * Generic SPI driver header file
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* --------------------------------------------------------------------------
  AUTHOR      : Daniel Allred
 --------------------------------------------------------------------------- */
 
#ifndef _SPI_TI_H_
#define _SPI_TI_H_

#include "tistdtypes.h"

// Prevent C++ name mangling
#ifdef __cplusplus
extern far "c" {
#endif

/************************************************************
* Global Macro Declarations                                 *
************************************************************/

#define SPI_RXINTFLAG     (0x00000100u)
#define SPI_OVRNINTFLG    (0x00000040u)
#define SPI_BITERRFLG     (0x00000010u)
#define SPI_DESELECTFLG   (0x00000004u)            // SPI_PARERRFLG

#define SPI_CS_NUM        (0)


/***********************************************************
* Global Typedef declarations                              *
***********************************************************/

typedef Uint32 T_SPI_PORT;

// SPI Module Roles
typedef enum _SPI_ROLE_
{
  SPI_ROLE_MASTER = 0x01,
  SPI_ROLE_SLAVE = 0x02
}
SPI_Role;

// SPI Module Modes of Operation
typedef enum _SPI_MODE_
{
  SPI_MODE_3PIN       = 0x01,
  SPI_MODE_4PIN_CS    = 0x02,
  SPI_MODE_4PIN_ENA   = 0x04,
  SPI_MODE_5PIN       = 0x08
}
SPI_Mode;

typedef struct _SPI_CONFIG_
{
  Uint8   polarity;
  Uint8   phase;
  Uint8   prescalar;
  Uint8   charLen;
  Uint32  delayAfterCSdown;
  Uint32  delayBeforeCSup;
  Uint32  delayAfterCSup;
}
SPI_Config;

// SPI driver structure
typedef struct _SPI_INFO_
{
  Uint32      peripheralNum;
  Uint32      csNum;
  void        *regs;
  SPI_Role    role;
  SPI_Mode    mode;
  SPI_Config  *config;
}
SPI_InfoObj, *SPI_InfoHandle;


/************************************************************
* Global Function Declarations                              *
************************************************************/

extern SPI_InfoHandle SPI_open(Uint32 const         spiPeripheralNum, Uint32 const  csNum,
                               SPI_Role const       role,        SPI_Mode const     mode,
                               SPI_Config* const    config);
extern Uint32 SPI_reset(       SPI_InfoHandle const hSPIInfo);
extern Uint32 SPI_readBytes(   SPI_InfoHandle const hSPIInfo,
                               Uint32 const         byteCnt,     void* const        dest);
extern Uint32 SPI_writeBytes(  SPI_InfoHandle const hSPIInfo,
                               Uint32 const         byteCnt,     void const* const  src);
extern Uint32 SPI_xferOneChar( SPI_InfoHandle const hSPIInfo,
                               Uint32 const         dataOut);
extern void SPI_enableCS(      SPI_InfoHandle const hSPIInfo);
extern void SPI_disableCS(     SPI_InfoHandle const hSPIInfo);
extern void SPI_sync(          SPI_InfoHandle const hSPIInfo,
                               Uint32 const         srcByteCnt,  void const* const  src,
                               Uint32 const         destByteCnt, void* const        dest );
extern Uint32 SPIBOOT_copy(    Uint32 const         spiInstanceNum);


/***********************************************************
* End file                                                 *
***********************************************************/

#ifdef __cplusplus
}
#endif

#endif //_SPI_TI_H_

