/*======================================================================================================================
  (c) Copyright OMRON Europe BV 2011
  All Right Reserved
  OMRON Development & Engineering
  -----------------------------------------------------------------------------------------------------------------*//**
  @file
  
  This file contains the U-boot header definitions used to determine if and process an image contains a valid U-boot
  header.
*//*--------------------------------------------------------------------------------------------------------------------
  Unique identifier generated by svn:
  $Header$

======================================================================================================================*/

#ifndef __UBOOT_HDR_H                         /* Only include this file once   */
#define __UBOOT_HDR_H                         /* If not defined, define it now */

/*=== Include files ==================================================================================================*/
#include "types.h"

/*=== Constant definitions ===========================================================================================*/
/**
 * Magic U-boot header number.
 */
#define IH_MAGIC      0x27051956

/**
 * The max length for the image name.
 */
#define IH_NMLEN      32

/*
 * Operating System Codes
 */
#define IH_OS_INVALID   0 /* Invalid OS */
#define IH_OS_OPENBSD   1 /* OpenBSD  */
#define IH_OS_NETBSD    2 /* NetBSD */
#define IH_OS_FREEBSD   3 /* FreeBSD  */
#define IH_OS_4_4BSD    4 /* 4.4BSD */
#define IH_OS_LINUX   5 /* Linux  */
#define IH_OS_SVR4    6 /* SVR4   */
#define IH_OS_ESIX    7 /* Esix   */
#define IH_OS_SOLARIS   8 /* Solaris  */
#define IH_OS_IRIX    9 /* Irix   */
#define IH_OS_SCO   10  /* SCO    */
#define IH_OS_DELL    11  /* Dell   */
#define IH_OS_NCR   12  /* NCR    */
#define IH_OS_LYNXOS    13  /* LynxOS */
#define IH_OS_VXWORKS   14  /* VxWorks  */
#define IH_OS_PSOS    15  /* pSOS   */
#define IH_OS_QNX   16  /* QNX    */
#define IH_OS_U_BOOT    17  /* Firmware */
#define IH_OS_RTEMS   18  /* RTEMS  */
#define IH_OS_ARTOS   19  /* ARTOS  */
#define IH_OS_UNITY   20  /* Unity OS */
#define IH_OS_INTEGRITY   21  /* INTEGRITY  */
#define IH_OS_OSE   22  /* OSE    */

/*
 * CPU Architecture Codes (supported by Linux)
 */
#define IH_ARCH_INVALID   0 /* Invalid CPU  */
#define IH_ARCH_ALPHA   1 /* Alpha  */
#define IH_ARCH_ARM   2 /* ARM    */
#define IH_ARCH_I386    3 /* Intel x86  */
#define IH_ARCH_IA64    4 /* IA64   */
#define IH_ARCH_MIPS    5 /* MIPS   */
#define IH_ARCH_MIPS64    6 /* MIPS  64 Bit */
#define IH_ARCH_PPC   7 /* PowerPC  */
#define IH_ARCH_S390    8 /* IBM S390 */
#define IH_ARCH_SH    9 /* SuperH */
#define IH_ARCH_SPARC   10  /* Sparc  */
#define IH_ARCH_SPARC64   11  /* Sparc 64 Bit */
#define IH_ARCH_M68K    12  /* M68K   */
#define IH_ARCH_MICROBLAZE  14  /* MicroBlaze   */
#define IH_ARCH_NIOS2   15  /* Nios-II  */
#define IH_ARCH_BLACKFIN  16  /* Blackfin */
#define IH_ARCH_AVR32   17  /* AVR32  */
#define IH_ARCH_ST200         18  /* STMicroelectronics ST200  */

/*
 * Image Types
 *
 * "Standalone Programs" are directly runnable in the environment provided by U-Boot; it is expected that (if they
 * behave well) you can continue to work in U-Boot after return from the Standalone Program. "OS Kernel Images" are
 * usually images of some Embedded OS which will take over control completely. Usually these programs will install
 * their own set of exception handlers, device drivers, set up the MMU, etc. - this means, that you cannot expect to
 * re-enter U-Boot except by resetting the CPU. "RAMDisk Images" are more or less just data blocks, and their parameters
 * (address, size) are passed to an OS kernel that is being started. "Multi-File Images" contain several images,
 * typically an OS(Linux) kernel image and one or more data images like RAMDisks. This construct is useful for instance
 * when you want to boot over the network using BOOTP etc., where the boot server provides just a single image file,
 * but you want to get for instance an OS kernel and a RAMDisk image.
 *
 * "Multi-File Images" start with a list of image sizes, each image size (in bytes) specified by an "uint32_t" in
 * network byte order. This list is terminated by an "(uint32_t)0". Immediately after the terminating 0 follow the
 * images, one by one, all aligned on "uint32_t" boundaries (size rounded up to a multiple of 4 bytes - except for the
 * last file).
 *
 * "Firmware Images" are binary images containing firmware (like U-Boot or FPGA images) which usually will be programmed
 * to flash memory.
 *
 * "Script files" are command sequences that will be executed by U-Boot's command interpreter; this feature is
 * especially useful when you configure U-Boot to use a real shell (hush) as command interpreter (=> Shell Scripts).
 */
#define IH_TYPE_INVALID   0 /* Invalid Image    */
#define IH_TYPE_STANDALONE  1 /* Standalone Program   */
#define IH_TYPE_KERNEL    2 /* OS Kernel Image    */
#define IH_TYPE_RAMDISK   3 /* RAMDisk Image    */
#define IH_TYPE_MULTI   4 /* Multi-File Image   */
#define IH_TYPE_FIRMWARE  5 /* Firmware Image   */
#define IH_TYPE_SCRIPT    6 /* Script file      */
#define IH_TYPE_FILESYSTEM  7 /* Filesystem Image (any type)  */
#define IH_TYPE_FLATDT    8 /* Binary Flat Device Tree Blob */
#define IH_TYPE_KWBIMAGE  9 /* Kirkwood Boot Image    */
#define IH_TYPE_IMXIMAGE  10  /* Freescale IMXBoot Image  */

/*
 * Compression Types
 */
#define IH_COMP_NONE    0 /*  No   Compression Used */
#define IH_COMP_GZIP    1 /* gzip  Compression Used */
#define IH_COMP_BZIP2   2 /* bzip2 Compression Used */
#define IH_COMP_LZMA    3 /* lzma  Compression Used */
#define IH_COMP_LZO   4 /* lzo   Compression Used */

#define IH_MAGIC  0x27051956  /* Image Magic Number   */
#define IH_NMLEN    32  /* Image Name Length    */

/*=== Macro definitions ==============================================================================================*/

/*=== Type definitions ===============================================================================================*/
/**
 * Complete definition of an U-boot image header.
 */
typedef struct {
  UNS32  magic;           /**< Image Header Magic Number  */
  UNS32  hcrc;            /**< Image Header CRC Checksum  */
  UNS32  time;            /**< Image Creation Timestamp */
  UNS32  size;            /**< Image Data Size    */
  UNS32  load;            /**< Data  Load  Address    */
  UNS32  ep;              /**< Entry Point Address    */
  UNS32  dcrc;            /**< Image Data CRC Checksum  */
  UNS8   os;              /**< Operating System   */
  UNS8   arch;            /**< CPU architecture   */
  UNS8   type;            /**< Image Type     */
  UNS8   comp;            /**< Compression Type   */
  UNS8   name[IH_NMLEN];  /**< Image Name   */
} PACKED_BYTE_ALIGN(T_UBOOT_IMG_HEADER);

/*=== Type definitions for events ====================================================================================*/

/*=== Macro-function definitions =====================================================================================*/

/*=== Inline-function definitions ====================================================================================*/
  
/*=== Function declarations (Interface functions) ====================================================================*/
 
 
#endif

