#!/bin/bash

# Parameters
PROJECT_ROOT=$1
RELEASE_VERSION=$2

# Determine relative or absolute path:
if [ -z "$(echo $PROJECT_ROOT | grep ^/)" ]; then
    PROJECT_ROOT=$(pwd)/$1
fi

# Print the header file introductory comment
echo "/* DO NOT EDIT: Automatically generated by '`basename $0`' */"

# 1. Print the failure to stderr and stdout in .h file format.
# 2. Then exit with an error code.
fail() {
	echo "#error $1"
	echo "`basename $0` error: $1" >&2
	exit 1
}

# Valid project root checks:
if ! [ -d $PROJECT_ROOT ]; then
	fail "$1 is not a valid directory"
fi

if ! pushd $PROJECT_ROOT >/dev/null; then
	fail "Could not cd to $1"
fi

if ! [ -f ".externals" ]; then
	popd
	fail "`pwd` is not managed by the ext tool"
fi

# Generate .h file output

get_git_version() {
  pushd $PROJECT_ROOT &>/dev/null
  GIT_VERSION=$(git describe --tags --always --long --dirty)
  
  VERSION=$(echo $GIT_VERSION | awk 'BEGIN {FS="-"} ; {print $1}')
  VERSION_GIT_REF=$(echo $GIT_VERSION | awk 'BEGIN {FS="-"} ; {print $3}')
  VERSION_IS_DIRTY=$(echo $GIT_VERSION | awk 'BEGIN {FS="-"} ; {print $4}')
  if [ ! -z "$VERSION_IS_DIRTY" ]; then
      VERSION_IS_DIRTY="++"
  fi
  popd &>/dev/null
}

get_git_version

echo "#define AUTO_FULL_VERSION   \"$GIT_VERSION\""
echo "#define AUTO_SOFT_VERSION   \"$VERSION\""
echo "#define AUTO_SOFT_GIT_REF   \"$VERSION_GIT_REF\""
echo "#define AUTO_SOFT_DIRTY     \"$VERSION_IS_DIRTY\""

popd >/dev/null
