/*======================================================================================================================
  (c) Copyright OMRON Europe BV 2011
  All Right Reserved
  OMRON Development & Engineering
  -----------------------------------------------------------------------------------------------------------------*//**
  @file

  Linux device driver for the Micrel KSZ8864 switch

  It is specifically designed by and for Omron to retrieve the link status (up/down, speed, etc). It is not a full
  switch driver for network access.
  @pre -
*//*--------------------------------------------------------------------------------------------------------------------
      Unique identifier generated by svn:
      $Header$

======================================================================================================================*/
#ifndef __ETHSW_DRV_H_
#define __ETHSW_DRV_H_

/*=== Include files ==================================================================================================*/
#include <stddef.h>		// bool

/*=== Constant definitions ===========================================================================================*/
#define GPIO_TO_PIN(bank, gpio)	(16 * (bank) + (gpio))

#define ETHSW_CHIP_ID                                  (0x95)

#define ETHSW_DEV_MAJOR                                (280)
#define ETHSW_DEV_MINOR                                (0)
#define ETHSW_NUM_MINORS                               (1)


#define ETHSW_SMI_RX_BUF_SIZE                          (32U)
#define ETHSW_SMI_TX_BUF_SIZE                          (32U)
#define KS_PROC_MAX_WRITE_SIZE                         ((unsigned long)512U)

#define KS_PROC_DIR                                    "ethsw"
#define KS_REGS_DIR                                    "regs"
#define KS_MII_DIR                                     "mii"
#define KS_CTRL_DIR                                    "control"
#define KS_MIB_DIR                                     "mib"
#define KS_INTERFACE_ETH                               "eth0"
#define PHY_ADDR                                       (7)
#define SWITCH_IRQ_NO                                  (23)
#define SWITCH_IRQ_PIN                                 (GPIO_TO_PIN(6, 3))
#define SWITCH_IRQ_NAME                                "ETH_IRQn"
#define ETH_RESET_PIN                                  (GPIO_TO_PIN(6, 2))
#define ETH_RESET_NAME                                 "ETH_RSTn"
#ifdef HAS_EXT_PHY
#define PHY_IRQ_NO                                     (23)
#define PHY_IRQ_PIN                                    (AT91_PIN_PB25)
#define PHY_IRQ_NAME                                   "PHY_IRQn"
#endif

#define AUTONEG_BUSY                                   ('0')
#define AUTONEG_DONE                                   ('1')

#define LINK_STATUS_NONE                               ('0')
#define LINK_STATUS_GOOD                               ('1')

#define SPEED_MODE_10MB_HALF                           ('0')
#define SPEED_MODE_10MB_FULL                           ('1')
#define SPEED_MODE_100MB_HALF                          ('2')
#define SPEED_MODE_100MB_FULL                          ('3')
#define SPEED_MODE_UNDEFINED                           ('9')

#define ETH_RESET_TIME_MS                              (20)
#define ETH_RESET_KEY                                  "d0r3s3t"
#define ETH_RESET_BAD_KEY                              "error: Wrong key"
#define ETH_RESET_DONE                                 "Reset done"
#define ETH_RESET_IDLE                                 "Write "ETH_RESET_KEY" to this file to reset the switch & PHY"
#define ETH_DMAC_FLUSH_KEY                             "f1ush"
#define ETH_DMAC_FLUSH_BAD_KEY                         "error: Wrong key"
#define ETH_DMAC_FLUSH_DONE                            "Flush done"
#define ETH_DMAC_FLUSH_IDLE                            "Write "ETH_DMAC_FLUSH_KEY" to this file to flush the dynamic MAC address table"

#define ETHSW_MAC_LEN									(6)

/* Return codes */
#define ETHSW_FAIL										(-1)
#define ETHSW_SUCCESS									(0)

/* Mapping of port function to switch port numbers */
#define MRP_PORT_0										(ETHSW_PORT1)
#define MRP_PORT_1										(ETHSW_PORT2)
#define MPU_PORT										(ETHSW_PORT4)
#define AUTOROUTE_PORT									(0)

/* Forwarding flags for static MAC table entries */
#define ETHSW_FORWARD_PORT_MRP1							(1U << ((int)MRP_PORT_0-(int)ETHSW_PORT1))
#define ETHSW_FORWARD_PORT_MRP2							(1U << ((int)MRP_PORT_1-(int)ETHSW_PORT1))
#define ETHSW_FORWARD_PORT_MPU							(1U << ((int)MPU_PORT-(int)ETHSW_PORT1))

/* Bitmasks for selecting static/dynamic MAC table entries */
#define ETHSW_MAC_TABLE_STATIC							(1U << 0)
#define ETHSW_MAC_TABLE_DYNAMIC							(1U << 1)

/*=== Macro definitions ==============================================================================================*/

/*=== Type definitions ===============================================================================================*/

typedef enum
{
	ETHSW_LINK_STATE_DOWN = 0,
	ETHSW_LINK_STATE_UP,
} T_ETHSW_LINK_STATE;

typedef enum
{
	ETHSW_SPEED_UNKNOWN = 0,
	ETHSW_SPEED_10MB_HALF,
	ETHSW_SPEED_10MB_FULL,
	ETHSW_SPEED_100MB_HALF,
	ETHSW_SPEED_100MB_FULL,
} T_ETHSW_LINK_SPEED;

typedef enum
{
	T_ETHSW_PORT_STATE_FORWARDING = 0,
	T_ETHSW_PORT_STATE_DISABLE,
	T_ETHSW_PORT_STATE_BLOCKING,
	T_ETHSW_PORT_STATE_LISTENING,
	T_ETHSW_PORT_STATE_LEARNING,
	T_ETHSW_PORT_STATE_MAX
} T_ETHSW_PORT_STATE;

typedef enum
{
	ETHSW_PORT1 = 1,
	ETHSW_PORT2 = 2,
	ETHSW_PORT3 = 3,
	ETHSW_PORT4 = 4,
} T_ETHSW_PORT;

/*=== Type definitions for events ====================================================================================*/

/*=== Macro-function definitions =====================================================================================*/

/*=== Inline-function definitions ====================================================================================*/

/*=== Function declarations (Interface functions) ====================================================================*/
extern int	ethsw_getLinkStatus				(const T_ETHSW_PORT port, T_ETHSW_LINK_STATE * const state);
extern int	ethsw_getLinkInfo				(const T_ETHSW_PORT port, T_ETHSW_LINK_SPEED * const speed);
extern int	ethsw_getMacAddr				(const T_ETHSW_PORT port, unsigned char * const mac);
extern void	ethsw_clearMacTable				(const unsigned tableMask);
extern int	ethsw_addStaticMacTableEntry	(unsigned char const * const mac, const unsigned forwardMask,
												const bool override );
extern int	ethsw_delStaticMacTableEntry	(unsigned char const * const mac);
extern int	ethsw_setPortState				(const T_ETHSW_PORT port, const T_ETHSW_PORT_STATE state);

#endif

