/*
 * Driver for the PHY in the Micrel Ethernet switches
 *
 * Fakes a 100Mbps Full-duplex status, which functionally is the case
 * when an EMAC is connected directly to a switch
 *
 * Author: Pieter Smith <pieter.smith@eu.omron.com>
 *
 * Copyright (C) 2012 Omron Europe B.V. - http://industrial.omron.eu/
 *
 * Derived from: drivers/net/phy/national.c
 *
 * Original Copyrights follow:
 *
 * Copyright (c) 2008 STMicroelectronics Limited
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/mii.h>
#include <linux/ethtool.h>
#include <linux/phy.h>
#include <linux/netdevice.h>

/* KSZ8864 phy identifier values */
#define KSZ8864_PHY_ID	0x00221450

static int micrel_switch_config_aneg(struct phy_device *phydev)
{
	return 0;
}

static int micrel_switch_read_status(struct phy_device *phydev)
{
	/* The KSZ8864 is a switch: The link is always good, full-duplex and 100 Mbps */

	phydev->link = 1;
	phydev->speed = SPEED_100;
	phydev->duplex = DUPLEX_FULL;
	phydev->pause = 0;
	phydev->asym_pause = 0;

	return 0;
}

static int micrel_switch_config_init(struct phy_device *phydev)
{
	/* Trigger the PHY framework to take the fixed link details into account
         * and disable link status polling */

//	phydev->state = PHY_CHANGELINK;
	phydev->state = PHY_RUNNING;
	phydev->irq = PHY_IGNORE_INTERRUPT;

	printk(KERN_ALERT "%s\n", __FUNCTION__);

	return 0;
}

static int micrel_switch_probe(struct phy_device *phydev)
{
	printk(KERN_ALERT "%s\n", __FUNCTION__);
}

static struct phy_driver ksz8864_driver = {
	.phy_id = KSZ8864_PHY_ID,
	.phy_id_mask = 0xfffffff0,
	.name = "Micrel KSZ8864",
	.features = PHY_BASIC_FEATURES,
//	.flags = PHY_HAS_INTERRUPT,
	.config_init = micrel_switch_config_init,
	.config_aneg = micrel_switch_config_aneg,
	.read_status = micrel_switch_read_status,
//	.driver = {.owner = THIS_MODULE,}
	.probe		= micrel_switch_probe,//KDEBUG
};

static int __init micrel_switch_init(void)
{
	printk(KERN_ALERT "%s\n", __FUNCTION__);
	return phy_driver_register(&ksz8864_driver, THIS_MODULE);
}

static void __exit micrel_switch_exit(void)
{
	phy_driver_unregister(&ksz8864_driver);
}

//module_phy_driver(ksz8864_driver);

MODULE_DESCRIPTION("Micrel Ethernet Switch PHY driver");
MODULE_AUTHOR("Pieter Smith");
MODULE_LICENSE("GPL");

module_init(micrel_switch_init);
module_exit(micrel_switch_exit);

static struct mdio_device_id __maybe_unused ksz8864_tbl[] = {
	{ KSZ8864_PHY_ID, 0xfffffff0 },
	{ }
};

MODULE_DEVICE_TABLE(mdio, ksz8864_tbl);
