/*
 * UIO IBC driver.
 *
 * (c) Copyright OMRON Europe BV 2012
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
#include <linux/gpio.h>
#include <linux/platform_device.h>
#include <linux/uio_driver.h>       /* uio_info */
//#include <mach/da8xx.h>             /* DA8XX_GPIO_BASE */
#include <linux/module.h>

#undef pr_fmt
#define pr_fmt(fmt) "uio_da8xx_gpio:" fmt

#define DRV_VERSION "0.01"

#define DA8XX_GPIO_BASE		0x01e26000

static struct uio_info uio_da8xx_gpio_info = {
        .name       = "uio_da8xx_gpio",
        .version    = "1",
        .irq_flags  = 0,
};

static struct resource uio_da8xx_gpio_resource[] = {
        {
                .name   = "uio_da8xx_gpio",
                .start  = DA8XX_GPIO_BASE,
                .end    = DA8XX_GPIO_BASE + SZ_4K - 1,
                .flags  = IORESOURCE_MEM,
        },
};

static struct platform_device uio_da8xx_gpio_dev = {
        .name          = "uio_pdrv_genirq",
        .id            = 0,
        .dev           = {
                           .platform_data = &uio_da8xx_gpio_info,
        },
        .num_resources = ARRAY_SIZE(uio_da8xx_gpio_resource),
        .resource      = uio_da8xx_gpio_resource,
};

static void __exit uio_da8xx_gpio_exit(void)
{
        platform_device_unregister(&uio_da8xx_gpio_dev);
}
module_exit(uio_da8xx_gpio_exit);

static int __init uio_da8xx_gpio_init(void)
{
        int err = 0;

        err = platform_device_register(&uio_da8xx_gpio_dev);
        if (err) {
                pr_err("%s: registration failed: %d\n", __func__, err);
                return err;
        }

        pr_info("%s: found da8xx gpio sub-system\n", __func__);
        return 0;
}
module_init(uio_da8xx_gpio_init);

MODULE_LICENSE("GPL v2");
MODULE_VERSION(DRV_VERSION);
MODULE_AUTHOR("Pieter Smith <pieter.smith@eu.omron.com>");

