/*
 * UIO IBC driver.
 *
 * (c) Copyright OMRON Europe BV 2012
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
#include <linux/gpio.h>
#include <linux/platform_device.h>
#include <linux/uio_driver.h>       /* uio_info */
#include <linux/clk.h>
//#include <mach/aemif.h>
//#include <mach/da8xx.h>
//#include "../../arch/arm/mach-davinci/include/mach/aemif.h"
//#include "../../arch/arm/mach-davinci/include/mach/da8xx.h"
#include <linux/module.h>

#define DA8XX_AEMIF_CTL_BASE	0x68000000

#define CONFIG_OMN_IBC_BASE_ADDRESS 0x64000000
#define CONFIG_OMN_IBC_CS 4
#define CONFIG_OMN_IBC_IRQ_GPIO_BANK 2
#define CONFIG_OMN_IBC_IRQ_GPIO_PIN 2
#define CONFIG_OMN_IBC_WSETUP 10
#define CONFIG_OMN_IBC_WSTROBE 38
#define CONFIG_OMN_IBC_WHOLD 18
#define CONFIG_OMN_IBC_RSETUP 10
#define CONFIG_OMN_IBC_RSTROBE 140
#define CONFIG_OMN_IBC_RHOLD 15
#define CONFIG_OMN_IBC_TA 15
#define CONFIG_OMN_IBC_BUSWIDTH 2

#undef pr_fmt
#define pr_fmt(fmt) "uio_ibc:" fmt

#define DRV_VERSION "1.00"

#define GPIO_TO_PIN(bank, gpio)	(16 * (bank) + (gpio))

#define RSC_SHM     (0)
#define RSC_CTL     (1)

/*
struct davinci_aemif_timing {
	u8	wsetup;
	u8	wstrobe;
	u8	whold;

	u8	rsetup;
	u8	rstrobe;
	u8	rhold;

	u8	ta;
};
*/
/**
 * UIO identifier for and IRQ trigger
 */
static struct uio_info uio_ibc_info = {
	.name       = "uio_ibc",
	.version    = "1",
	.irq_flags  = IRQF_TRIGGER_FALLING,
};

/**
 * Setup of IBC shared memory 
 */
static struct resource uio_ibc_shm[] = {
	{
		.name   = "ibc_shm",
		.start  = CONFIG_OMN_IBC_BASE_ADDRESS,
		.end    = CONFIG_OMN_IBC_BASE_ADDRESS + SZ_64K - 1,
		.flags  = IORESOURCE_MEM,
	},
	{
		.start	= DA8XX_AEMIF_CTL_BASE,
		.end	= DA8XX_AEMIF_CTL_BASE + SZ_32K - 1,
		.flags	= IORESOURCE_MEM,
	},
};

/**
 * Setup of the IBC interrupt source
 */
static struct platform_device uio_ibc_dev = {
	.name          = "uio_pdrv_genirq",
	.id            = 2,
	.dev           = {
	                   .platform_data = &uio_ibc_info,
	},
	.num_resources = ARRAY_SIZE(uio_ibc_shm),
	.resource      = uio_ibc_shm,
};

#if defined(CONFIG_OMN_OVERRIDE_BOOTLOADER)
/**
 * AEMIF timing settings for the IBC in asynchronous addr/data mode.
 */
static struct davinci_aemif_timing uio_ibc_timing = {
	.wsetup		= CONFIG_OMN_IBC_WSETUP,
	.wstrobe	= CONFIG_OMN_IBC_WSTROBE,
	.whold		= CONFIG_OMN_IBC_WHOLD,
	.rsetup		= CONFIG_OMN_IBC_RSETUP,
	.rstrobe	= CONFIG_OMN_IBC_RSTROBE,
	.rhold		= CONFIG_OMN_IBC_RHOLD,
	.ta		= CONFIG_OMN_IBC_TA,
};
#endif

static void __exit uio_ibc_exit(void)
{
	platform_device_unregister(&uio_ibc_dev);
}
module_exit(uio_ibc_exit);

static int __init uio_ibc_init(void)
{
    int ret;
	int err = 0;
    void __iomem *ctl;

	uio_ibc_info.irq = gpio_to_irq(GPIO_TO_PIN(
		CONFIG_OMN_IBC_IRQ_GPIO_BANK,
		CONFIG_OMN_IBC_IRQ_GPIO_PIN ));

	err = platform_device_register(&uio_ibc_dev);
	if (err) {
		pr_err("%s: registration failed: %d\n", __func__, err);
		return err;
	}

#if defined(CONFIG_OMN_OVERRIDE_BOOTLOADER)
    /* The da8xx must offset the CS bank with -2 */
	ctl = ioremap(uio_ibc_shm[RSC_CTL].start, resource_size(&uio_ibc_shm[RSC_CTL]));
    ret = davinci_aemif_setup_timing(&uio_ibc_timing,
            ctl, CONFIG_OMN_IBC_CS-2);
    if (ret < 0) {
        pr_err("%s: IBC timing values setup fail: %d\n", __func__, ret);
        return -EINVAL;
    }
#endif

	pr_info("%s: found IBC asic\n", __func__);
	return 0;
}
module_init(uio_ibc_init);

MODULE_LICENSE("GPL v2");
MODULE_VERSION(DRV_VERSION);
MODULE_AUTHOR("Pieter Smith <pieter.smith@eu.omron.com>");

